@@ -7,10 +7,6 @@
 #import "NJBQueueItem.h"
 #import "defs.h"
 
-
-/* todo: ask to close app when close main window
- */
-
 @implementation MainController
 
 // init/dealloc methods
@@ -60,6 +56,16 @@
 	return settingsTab;
 }
 
+- (DuplicateItemsTab *)duplicateItemsTab
+{
+	return duplicateItemsTab;
+}
+
+- (InfoTab *)infoTab
+{
+	return infoTab;
+}
+
 /**********************/
 
 - (void)awakeFromNib
@@ -111,6 +117,8 @@
 	int tabIndex = [tabViewMain indexOfTabViewItem:[tabViewMain selectedTabViewItem]];
 	if (tabIndex == TAB_MUSIC)
 		[musicTab drawerWriteTag:modifiedTrack];
+	else if (tabIndex == TAB_DUPLICATES)
+		[duplicateItemsTab drawerWriteTag:modifiedTrack];
 }
 
 - (void)onFirstResponderChange:(NSResponder *)aResponder
@@ -171,17 +179,16 @@
 	[queueTab setNJBQueueConsumer:njbQueue];
 	[dataTab setNJBQueueConsumer:njbQueue];
 	[playlistsTab setNJBQueueConsumer:njbQueue];
+	[duplicateItemsTab setNJBQueueConsumer:njbQueue];
 	[settingsTab setNJBQueueConsumer:njbQueue];
+	[infoTab setNJBQueueConsumer:njbQueue];
 }
 
 - (NSApplicationTerminateReply)applicationShouldTerminate:(NSApplication *)sender
 {
-	if (![status isIdle])
+	if (![self shouldClose])
 	{
-		int result = NSRunAlertPanel(@"Quit XNJB", @"There are still items in the queue.  Are you sure you want to quit?  This could crash your NJB and/or XNJB.",
-																 @"No", @"Yes", nil);
-		if (result == NSAlertDefaultReturn)
-			return NSTerminateCancel;
+		return NSTerminateCancel;
 	}
 	// do it in this thread as want to block
 	[theNJB disconnect];
@@ -193,6 +200,18 @@
 	return NSTerminateNow;
 }
 
+- (BOOL)shouldClose
+{
+	if (![status isIdle])
+	{
+		int result = NSRunAlertPanel(@"Quit XNJB", @"There are still items in the queue.  Are you sure you want to quit?  This could crash your NJB and/or XNJB.",
+																 @"No", @"Yes", nil);
+		if (result == NSAlertDefaultReturn)
+			return NO;
+	}
+	return YES;
+}
+
 - (id)currentTab
 {
 	return [self tabAtIndex:[tabViewMain indexOfTabViewItem:[tabViewMain selectedTabViewItem]]];
@@ -219,6 +238,12 @@
 		case TAB_SETTINGS:
 			return settingsTab;
 			break;
+		case TAB_DUPLICATES:
+			return duplicateItemsTab;
+			break;
+		case TAB_INFO:
+			return infoTab;
+			break;
 		case TAB_QUEUE:
 			return queueTab;
 			break;
@@ -244,9 +269,9 @@
 		if ([currentItem displayStatus])
 		{
 			if ([result class] == [NJBTransactionResult class])
-				[status taskComplete:[result success]];
+				[status taskComplete:result];
 			else
-				[status taskComplete:YES];
+				[status taskComplete:[[NJBTransactionResult alloc] initWithSuccess:YES]];
 		}
 	}
 	[myQueueLock unlock];
@@ -264,6 +289,7 @@
 {
 	NJBQueueItem *connect = [[NJBQueueItem alloc] initWithTarget:theNJB withSelector:@selector(connect)];
 	[connect setStatus:STATUS_CONNECTING];
+	
 	[queueTab addItem:connect withSubItems:nil withDescription:@"Connecting to NJB"];
 	
 	[[njbQueue lock] lock];
@@ -342,4 +368,14 @@
 	[connectButton setTitle:@"Disconnect"];
 }
 
+/* called when the main window is closing
+ */
+- (BOOL)windowShouldClose:(id)sender
+{
+	[NSApp terminate:sender];
+	// don't close the window, because if we said don't close the app, we don't want the window closed but app still running
+	// if user asked to terminate app rather than just close window, windowShouldClose: is not called so we will close anyway
+	return NO;
+}
+
 @end
