@@ -36,12 +36,9 @@
 {
 	if (self = [super init])
 	{
-		previousTrackIndexSelected = -1;
 		
 		fileExtensionsToCodecsDictionary = [[NSDictionary alloc] initWithObjectsAndKeys:[NSNumber numberWithInt:CODEC_MP3],
 			@"mp3", [NSNumber numberWithInt:CODEC_WMA], @"wma", [NSNumber numberWithInt:CODEC_WAV], @"wav", nil];
-		
-		trackListIsUpToDate = NO;
 	}
 	return self;
 }
@@ -52,13 +49,6 @@
 	[super dealloc];
 }
 
-// accessor methods
-
-- (BOOL)trackListIsUpToDate
-{
-	return trackListIsUpToDate;
-}
-
 /**********************/
 
 - (void)awakeFromNib
@@ -69,55 +59,13 @@
 	[browser setBaseLocation:[preferences musicTabDir]];
 	
 	[njbTable setAllowedExtensions:extensions];
-	[njbTable setAutosaveName:PREF_KEY_MUSIC_TAB_TABLE];
-	[njbTable setAutosaveTableColumns:YES];
 }
 
 - (void)onConnectAndActive
 {
 	[super onConnectAndActive];
-	if ([playlistsTab trackListIsUpToDate] && [playlistsTab fullItemArray])
-	{
-		// playlistsTab has already got it
-		[fullItemArray release];
-		fullItemArray = [[MyMutableArray alloc] initWithMyMutableArray:[playlistsTab fullItemArray]];
-		[self clearSortingArrows];
-		[self searchNJBTable];
-		
-		trackListIsUpToDate = YES;
-	}
-	else
-	{
+	if (!trackListUpToDate)
 		[self loadTracks];
-		trackListIsUpToDate = YES;
-	}
-}
-
-- (SEL)sortSelectorForColumn:(NSString *)columnID
-{
-	if ([columnID isEqual:COLUMN_TITLE])
-		return [Track compareByTitleSelector];
-	else if ([columnID isEqual:COLUMN_ARTIST])
-		return [Track compareByArtistSelector];
-	else if ([columnID isEqual:COLUMN_ALBUM])
-		return [Track compareByAlbumSelector];
-	else if ([columnID isEqual:COLUMN_GENRE])
-		return [Track compareByGenreSelector];
-	else if ([columnID isEqual:COLUMN_LENGTH])
-		return [Track compareByLengthSelector];
-	else if ([columnID isEqual:COLUMN_TRACK_NO])
-		return [Track compareByTrackNumberSelector];
-	else if ([columnID isEqual:COLUMN_CODEC])
-		return [Track compareByCodecSelector];
-	else if ([columnID isEqual:COLUMN_FILESIZE])
-		return [Track compareByFilesizeSelector];
-	else if ([columnID isEqual:COLUMN_YEAR])
-		return [Track compareByYearSelector];
-	else
-	{
-		NSLog(@"Invalid column %@ in sortSelectorForColumn!", columnID);
-		return nil;
-	}
 }
 
 /* replaces track variables (%t, %a, ...)
@@ -186,9 +134,9 @@
 /* copy the item at index in itemArrayDisplaying
  * from the NJB
  */
-- (void)copyFileFromNJB:(int)index
+- (void)copyFileFromNJB:(MyItem *)item
 {
-	Track *track = (Track *)[[self itemArrayDisplaying] objectAtIndex:index];
+	Track *track = (Track *)item;
 	NSFileManager *manager = [NSFileManager defaultManager];
 	
 	NSString *filename = [self filenameForTrack:track];
@@ -211,7 +159,6 @@
 			// this is not the filename, need to make a directory
 			[dirsSoFar appendString:component];
 			[dirsSoFar appendString:@"/"];
-			NSLog(@"making directory %@", dirsSoFar);
 			// create the directory, will fail if already made, don't worry
 			[manager createDirectoryAtPath:dirsSoFar attributes:nil];
 		}
@@ -220,16 +167,8 @@
 	NSString *extension = [[NSString stringWithString:@"."] stringByAppendingString:[[track codecString] lowercaseString]];
 	NSString *fullPath = [directory stringByAppendingString:filename];
 	
-	while ([manager fileExistsAtPath:[fullPath stringByAppendingString:extension]])
-	{
-		NSLog(@"file exists: %@%@", fullPath, extension);
-		fullPath = [fullPath stringByAppendingString:@"_"];
-	}
-	
 	fullPath = [fullPath stringByAppendingString:extension];
 	
-	NSLog(@"downloading track %@ to %@", [track title], fullPath);
-	
 	[track setFullPath:fullPath];
 		
 	[self copyTrackFromNJB:track];
@@ -247,44 +186,33 @@
 	[copyTrack setStatus:STATUS_DOWNLOADING_TRACK];
 	NSString *description = [NSString stringWithFormat:@"Downloading track '%@'", [track title]];
 	
-	// tell PlaylistsTab
-	NJBQueueItem *updatePlaylistsTab = [[NJBQueueItem alloc] initWithTarget:playlistsTab withSelector:@selector(updateTracks)];
-	[updatePlaylistsTab setRunInMainThread:NO];
-	[updatePlaylistsTab setDisplayStatus:NO];
-	
 	// could update browser here, but causes browser to lose selection so annoying
 	
 	// write track tag to file
 	if ([preferences writeTagAfterCopy] && [track codec] == CODEC_MP3)
 	{
 		NJBQueueItem *writeTag = [[NJBQueueItem alloc] initWithTarget:self withSelector:@selector(writeTagToFile:) withObject:track];
-		// putting this to NO causes weird and wonderful crashes later on e.g. memory of the njb_t struct in theNJB getting overwritten...
-		// should be YES though (wasn't pre 0.23)
 		[writeTag setRunInMainThread:YES];
 		[writeTag setDisplayStatus:NO];
 
-		[self addToQueue:copyTrack withSubItems:[NSArray arrayWithObjects:updatePlaylistsTab, writeTag, nil] withDescription:description];
+		[self addToQueue:copyTrack withSubItems:[NSArray arrayWithObjects:writeTag, nil] withDescription:description];
 		[writeTag release];
 	}
 	else
-		[self addToQueue:copyTrack withSubItems:[NSArray arrayWithObjects:updatePlaylistsTab, nil] withDescription:description];
+		[self addToQueue:copyTrack withSubItems:nil withDescription:description];
 	
 	[copyTrack release];
-	[updatePlaylistsTab release];
 }
 
 /* copy the file at filename to
- * the NJB
+ * the NJB.
  */
 - (void)copyFileToNJB:(NSString *)filename
 {
-	ID3Tagger *tagger = [[ID3Tagger alloc] init];
-	[tagger setFileExtensionsToCodecsDictionary:fileExtensionsToCodecsDictionary];
-	[tagger setFilename:filename];
-	Track *track = [tagger readTrack];
+	Track *track = [[Track alloc] init];
 	[track setFullPath:filename];
 	[self copyTrackToNJB:track];
-	[tagger release];
+	[track release];
 }
 
 /* actually add the track to the queue
@@ -294,23 +222,33 @@
 {
 	if (![theNJB isConnected])
 		return;
-	NJBQueueItem *copyTrack = [[NJBQueueItem alloc] initWithTarget:theNJB withSelector:@selector(uploadTrack:) withObject:track];
+	NJBQueueItem *copyTrack = [[NJBQueueItem alloc] initWithTarget:self withSelector:@selector(uploadTrack:) withObject:track];
 	[copyTrack setStatus:STATUS_UPLOADING_TRACK];
 	
-	NJBQueueItem *addToList = [[NJBQueueItem alloc] initWithTarget:self withSelector:@selector(addToFullItemArray:) withObject:track withRunInMainThread:YES];
+	NJBQueueItem *addToList = [[NJBQueueItem alloc] initWithTarget:self withSelector:@selector(addToArray:) withObject:track withRunInMainThread:YES];
 	[addToList setDisplayStatus:NO];
 	
-	// tell PlaylistsTab
-	NJBQueueItem *updatePlaylistsTab = [[NJBQueueItem alloc] initWithTarget:playlistsTab withSelector:@selector(updateTracks)];
-	[updatePlaylistsTab setRunInMainThread:NO];
-	[updatePlaylistsTab setDisplayStatus:NO];
+	[copyTrack cancelItemIfFail:addToList];
 	
-	NSString *description = [NSString stringWithFormat:@"Uploading track '%@'", [track title]];
-	[self addToQueue:copyTrack withSubItems:[NSArray arrayWithObjects:addToList, updatePlaylistsTab, nil] withDescription:description];
+	NSString *description = [NSString stringWithFormat:@"Uploading track '%@'", [track fullPath]];
+	[self addToQueue:copyTrack withSubItems:[NSArray arrayWithObjects:addToList, nil] withDescription:description];
 	
 	[copyTrack release];
 	[addToList release];
-	[updatePlaylistsTab release];
+}
+
+/* uploads the track to the jukebox, run in worker thread
+ */
+- (NJBTransactionResult *)uploadTrack:(Track *)track
+{
+	ID3Tagger *tagger = [[ID3Tagger alloc] init];
+	[tagger setFileExtensionsToCodecsDictionary:fileExtensionsToCodecsDictionary];
+	[tagger setFilename:[track fullPath]];
+	Track *trackWithTag = [tagger readTrack];
+	[track setValuesToTrack:trackWithTag];
+	[tagger release];
+	
+	return [theNJB uploadTrack:track];
 }
 
 /* write the info in track to the tag in the file
@@ -322,67 +260,19 @@
 	[tagger release];
 }
 
-- (id)tableView:(NSTableView *)aTableView
-   objectValueForTableColumn:(NSTableColumn *)aTableColumn
-						row:(int)rowIndex
-{	
-	NSString *ident = [aTableColumn identifier];
-	Track *track = (Track *)[[self itemArrayDisplaying] objectAtIndex:rowIndex];
-	if ([ident isEqual:COLUMN_TITLE])
-		return [track title];
-	else if ([ident isEqual:COLUMN_ALBUM])
-		return [track album];
-	else if ([ident isEqual:COLUMN_ARTIST])
-		return [track artist];
-	else if ([ident isEqual:COLUMN_GENRE])
-		return [track genre];
-	else if ([ident isEqual:COLUMN_LENGTH])
-		return [track lengthString];
-	else if ([ident isEqual:COLUMN_TRACK_NO])
-		return [NSString stringWithFormat:@"%d", [track trackNumber]];
-	else if ([ident isEqual:COLUMN_CODEC])
-		return [track codecString];
-	else if ([ident isEqual:COLUMN_FILESIZE])
-		return [track filesizeString];
-	else if ([ident isEqual:COLUMN_YEAR])
-		return [NSString stringWithFormat:@"%d", [track year]];
-	else
-	{
-		NSLog(@"Invalid column tag in MusicTab");
-		return nil;
-	}
-}
-
-- (void)tableView:(NSTableView *)tableView didClickTableColumn:(NSTableColumn *)tableColumn
-{	
-	// unchanged tag edit will be lost
-	
-	// disable and clear the drawer
-	[drawerController clearAll];
-	[drawerController disableAll];
-	
-	[super tableView:tableView didClickTableColumn:tableColumn];
-}
-
 /* update the drawer when someone selects a different
  * track
  */
 - (void)tableViewSelectionDidChange:(NSNotification *)aNotification
 {
+	// we might be called from code changing the selection
+	if (activeObject != OBJECT_NJB_TABLE)
+		return;
 	[super tableViewSelectionDidChange:aNotification];
 	// unchanged tag edit will be lost
 	
-	int newRow = [njbTable selectedRow];
-	previousTrackIndexSelected = newRow;
-	if (newRow == -1)
-		return;
-	
-	objectToSaveTrack = OBJECT_NJB_TABLE;
-	
-	// update display to show new track
-	[drawerController showTrack:(Track *)[[self itemArrayDisplaying] objectAtIndex:newRow]];
-	if (![theNJB isConnected])
-		[drawerController disableWrite];
+	if ([njbTable selectedRow] != -1)
+		objectToSaveTrack = OBJECT_NJB_TABLE;
 }
 
 /* called by MainController when we must write
@@ -393,44 +283,31 @@
 	switch (objectToSaveTrack)
 	{
 		case OBJECT_NJB_TABLE:
-			NSLog(@"Saving to njb");
-			int rowIndex = [njbTable selectedRow];
 			// none selected, shouldn't be here anyway
-			if (rowIndex == -1)
+			if ([njbTable selectedRow] == -1)
 				return;	
 			
-			NJBQueueItem *changeTrackTag = [[NJBQueueItem alloc] initWithTarget:theNJB withSelector:@selector(changeTrackTag:) withObject:modifiedTrack];
-			[changeTrackTag setStatus:STATUS_UPDATING_TRACK_TAG];
+			Track *oldTrack = (Track *)[[arrayController arrangedObjects] objectAtIndex:[njbTable selectedRow]];
 			
-			Track *oldTrack = (Track *)[[self itemArrayDisplaying] objectAtIndex:rowIndex];
+			NJBQueueItem *changeTrackTag = [[NJBQueueItem alloc] initWithTarget:theNJB withSelector:@selector(changeTrackTagTo:from:) withObject:modifiedTrack];
+			[changeTrackTag setObject2:oldTrack];
+			[changeTrackTag setStatus:STATUS_UPDATING_TRACK_TAG];
 			
 			NJBQueueItem *updateTrackList = [[NJBQueueItem alloc] initWithTarget:self withSelector:@selector(replaceTrack:withTrack:)
 																																withObject:oldTrack withRunInMainThread:YES];
 			[updateTrackList setObject2:modifiedTrack];
 			[updateTrackList setDisplayStatus:NO];
-
-			// call searchNJBTable to pretend the search list changed to update
-			// saves data and does a reload
-			NJBQueueItem *updateDisplay = [[NJBQueueItem alloc] initWithTarget:self withSelector:@selector(searchNJBTable)];
-			[updateDisplay setRunInMainThread:YES];
-			[updateDisplay setDisplayStatus:NO];
 			
-			// tell PlaylistsTab
-			NJBQueueItem *updatePlaylistsTab = [[NJBQueueItem alloc] initWithTarget:playlistsTab withSelector:@selector(updateTracks)];
-			[updatePlaylistsTab setRunInMainThread:NO];
-			[updatePlaylistsTab setDisplayStatus:NO];
+			[changeTrackTag cancelItemIfFail:updateTrackList];
 			
 			NSString *description = [NSString stringWithFormat:@"Changing tag for track '%@'", [oldTrack title]];
-			[self addToQueue:changeTrackTag withSubItems:[NSArray arrayWithObjects:updateTrackList, updateDisplay, updatePlaylistsTab, nil] withDescription:description];
+			[self addToQueue:changeTrackTag withSubItems:[NSArray arrayWithObjects:updateTrackList, nil] withDescription:description];
 
 			[changeTrackTag release];
 			[updateTrackList release];
-			[updateDisplay release];
-			[updatePlaylistsTab release];
 			
 			break;
 		case OBJECT_FILE_BROWSER:
-			NSLog(@"Saving to browser, filename %@", [modifiedTrack fullPath]);
 			[self writeTagToFile:modifiedTrack];
 			break;
 		default:
@@ -438,41 +315,6 @@
 	}
 }
 
-- (void)searchNJBTable
-{
-	// will mess up indexes, don't want to save anyway
-	previousTrackIndexSelected = -1;
-	
-	NSString *searchString = [njbSearchField stringValue];
-	// release old track list
-	[truncatedItemArray release];
-	
-	if ([searchString length] == 0)
-	{
-		truncatedItemArray = nil;
-		[njbTable reloadData];
-		return;
-	}
-	
-	truncatedItemArray = [fullItemArray searchFor:searchString];
-	[truncatedItemArray retain];
-	
-	[njbTable reloadData];
-	if ([njbTable selectedRow] != -1)
-	{
-		// update display to show new track
-		objectToSaveTrack = OBJECT_NJB_TABLE;
-		[drawerController showTrack:(Track *)[[self itemArrayDisplaying] objectAtIndex:[njbTable selectedRow]]];
-		if (![theNJB isConnected])
-			[drawerController disableWrite];
-	}
-	else
-	{
-		[drawerController clearAll];
-		[drawerController disableAll];
-	}	
-}
-
 - (void)fileSystemBrowserFileClick:(NSString *)path
 {
 	[super fileSystemBrowserFileClick:path];
@@ -501,13 +343,6 @@
 	if (aResponder == njbTable)
 	{
 		objectToSaveTrack = OBJECT_NJB_TABLE;
-		// update display to show new track
-		if ([njbTable selectedRow] > -1)
-		{
-			[drawerController showTrack:(Track *)[[self itemArrayDisplaying] objectAtIndex:[njbTable selectedRow]]];
-			if (![theNJB isConnected])
-				[drawerController disableWrite];
-		}
 	}
 	else if ([aResponder class] == [NSMatrix class])
 		objectToSaveTrack = OBJECT_FILE_BROWSER;
@@ -522,26 +357,40 @@
  */
 - (void)loadTracks
 {
-	NJBQueueItem *clearSortingArrows = [[NJBQueueItem alloc] initWithTarget:self
-																														 withSelector:@selector(clearSortingArrows)
-																															 withObject:nil
-																											withRunInMainThread:YES];
-	[clearSortingArrows setDisplayStatus:NO];
+	if (trackListUpToDate)
+	{
+		NSLog(@"loadTracks called in Music tab when track list up to date");
+		return;
+	}
+	NSMutableArray *cachedTrackList = [theNJB cachedTrackList];
+	if (cachedTrackList != nil)
+	{
+		[tempArray release];
+		tempArray = cachedTrackList;
+		[tempArray retain];
+		[self showNewArray];
+		
+		trackListUpToDate = YES;
+		return;
+	}
 	
 	NJBQueueItem *getTracks = [[NJBQueueItem alloc] initWithTarget:self withSelector:@selector(downloadTrackList)];
 	[getTracks setStatus:STATUS_DOWNLOADING_TRACK_LIST];
 
 	NJBQueueItem *updateTable = [[NJBQueueItem alloc] initWithTarget:self
-																											withSelector:@selector(searchNJBTable)
+																											withSelector:@selector(showNewArray)
 																												withObject:nil
 																							 withRunInMainThread:YES];
 	[updateTable setDisplayStatus:NO];
 	
+	[getTracks cancelItemIfFail:updateTable];
+	
 	NSString *description = @"Getting track list";
-	[self addToQueue:getTracks withSubItems:[NSArray arrayWithObjects:clearSortingArrows, updateTable, nil] withDescription:description];
+	[self addToQueue:getTracks withSubItems:[NSArray arrayWithObjects:updateTable, nil] withDescription:description];
 	[getTracks release];
-	[clearSortingArrows release];
 	[updateTable release];
+	// it's not actually yet, but will be in a while, don't want to go copying anything
+	trackListUpToDate = YES;
 }
 
 /* this is called from the queue consumer 
@@ -550,13 +399,13 @@
  */
 - (NJBTransactionResult *)downloadTrackList
 {
-	NSLog(@"Getting tracks");
-	[fullItemArray release];
-	fullItemArray = [theNJB tracks];
+	[tempArray release];
+	tempArray = [theNJB tracks];
+	[tempArray retain];
 	
-	NJBTransactionResult *result = [[NJBTransactionResult alloc] initWithSuccess:(fullItemArray != nil)];
+	NJBTransactionResult *result = [[NJBTransactionResult alloc] initWithSuccess:(tempArray != nil)];
 	
-	[fullItemArray retain];
+	trackListUpToDate = YES;
 	
 	return [result autorelease];
 }
@@ -566,62 +415,52 @@
 	Track *track = (Track *)item;
 
 	NJBQueueItem *deleteTrack = [[NJBQueueItem alloc] initWithTarget:theNJB withSelector:@selector(deleteTrack:)
-																												withObject:([NSNumber numberWithInt:[track trackID]])];
+																												withObject:track];
 	[deleteTrack setStatus:STATUS_DELETING_TRACK];
 		
-	NJBQueueItem *removeFromAllTracks = [[NJBQueueItem alloc] initWithTarget:fullItemArray withSelector:@selector(removeObject:)
+	NJBQueueItem *removeFromAllTracks = [[NJBQueueItem alloc] initWithTarget:self withSelector:@selector(removeFromArray:)
 																																	withObject:track];
 	[removeFromAllTracks setRunInMainThread:YES];
 	[removeFromAllTracks setDisplayStatus:NO];
-		
-	// call searchNJBTable to pretend the search list changed to update
-	// saves data and does a reload
-	NJBQueueItem *updateTrackList = [[NJBQueueItem alloc] initWithTarget:self withSelector:@selector(searchNJBTable)];
-	[updateTrackList setRunInMainThread:YES];
-	[updateTrackList setDisplayStatus:NO];
 	
-	// tell PlaylistsTab
-	NJBQueueItem *updatePlaylistsTab = [[NJBQueueItem alloc] initWithTarget:playlistsTab withSelector:@selector(updateTracks)];
-	[updatePlaylistsTab setRunInMainThread:NO];
-	[updatePlaylistsTab setDisplayStatus:NO];
+	[deleteTrack cancelItemIfFail:removeFromAllTracks];
 	
 	NSString *description = [NSString stringWithFormat:@"Deleting track '%@'", [track title]];
-	[self addToQueue:deleteTrack withSubItems:[NSArray arrayWithObjects:removeFromAllTracks, updateTrackList, updatePlaylistsTab, nil] withDescription:description];
+	[self addToQueue:deleteTrack withSubItems:[NSArray arrayWithObjects:removeFromAllTracks, nil] withDescription:description];
 
 	[deleteTrack release];
 	[removeFromAllTracks release];
-	[updateTrackList release];
-	[updatePlaylistsTab release];
-		
-	NSLog(@"Deleting track id %d with title %@", [track trackID], [track title]);		
 }
 
 - (void)replaceTrack:(Track *)replace withTrack:(Track *)new
 {
-	[fullItemArray replaceObject:replace withObject:new];
+	[arrayController replaceObject:replace withObject:new];
+//	[arrayController rearrangeObjects];
+}
+
+- (void)NJBConnected:(NSNotification *)note
+{
+	trackListUpToDate = NO;
+	
+	[super NJBConnected:note];
 }
 
 - (void)NJBDisconnected:(NSNotification *)note
 {
+	trackListUpToDate = NO;
+	
 	[super NJBDisconnected:note];
 	
 	if (isActive && activeObject == OBJECT_NJB_TABLE)
 		[drawerController disableWrite];
-	
-	trackListIsUpToDate = NO;
 }
 
 - (void)activate
 {
 	[super activate];
-	if (activeObject == OBJECT_NJB_TABLE)
+	if ([theNJB isConnected] && !trackListUpToDate)
 	{
-		if ([njbTable selectedRow] != -1)
-		{
-			[drawerController showTrack:(Track *)[[self itemArrayDisplaying] objectAtIndex:[njbTable selectedRow]]];
-			if (![theNJB isConnected])
-				[drawerController disableWrite];
-		}
+		[self loadTracks];
 	}
 	else if (activeObject == OBJECT_FILE_BROWSER)
 	{
@@ -642,4 +481,28 @@
 	[super applicationTerminating:note];
 }
 
+- (void)NJBTrackListModified:(NSNotification *)note
+{
+	if (!isActive)
+	{
+		trackListUpToDate = NO;
+	}
+}
+
+- (void)showDrawerInfo
+{
+	// update display to show new track
+	if ([njbTable selectedRow] == -1)
+	{
+		[drawerController clearAll];
+		[drawerController disableAll];
+	}
+	else
+	{
+		[drawerController showTrack:(Track *)[[arrayController arrangedObjects] objectAtIndex:[njbTable selectedRow]]];
+		if (![theNJB isConnected])
+			[drawerController disableWrite];
+	}
+}
+
 @end
\ No newline at end of file
