@@ -6,7 +6,6 @@
 #import <Cocoa/Cocoa.h>
 #import "MyTableTab.h"
 #import "FileSystemBrowser.h"
-#import "MyMutableArray.h"
 #import "MyItem.h"
 
 typedef enum _activeObjects {OBJECT_NJB_TABLE = 0, OBJECT_FILE_BROWSER = 1, OBJECT_OTHER = 2} activeObjects;
@@ -36,7 +35,7 @@ typedef enum _activeObjects {OBJECT_NJB_TABLE = 0, OBJECT_FILE_BROWSER = 1, OBJE
 - (void)deleteFromNJB;
 - (void)deleteFromFileSystem;
 - (void)copyFileToNJB:(NSString *)filename;
-- (void)copyFileFromNJB:(int)index;
+- (void)copyFileFromNJB:(MyItem *)item;
 - (void)deleteFromFileSystem;
 - (void)deleteFromNJB:(MyItem *)item;
 - (void)disableTransfers;
@@ -49,5 +48,8 @@ typedef enum _activeObjects {OBJECT_NJB_TABLE = 0, OBJECT_FILE_BROWSER = 1, OBJE
 - (void)applicationTerminating:(NSNotification *)note;
 - (NSString *)replaceBadFilenameChars:(NSString *)filename;
 - (BOOL)canDelete;
+- (void)addToArray:(MyItem *)item;
+- (void)removeFromArray:(MyItem *)item;
+- (void)showDrawerInfo;
 
 @end
