@@ -19,8 +19,6 @@
 {
 	if (self = [super init])
 	{
-		currentColumnID = nil;
-		
 		NSNotificationCenter *nc;
 		nc = [NSNotificationCenter defaultCenter];
 		[nc addObserver:self selector:@selector(applicationTerminating:) name:NOTIFICATION_APPLICATION_TERMINATING object:nil];
@@ -58,6 +56,7 @@
 		return @"";
 	NSMutableString *file = [[NSMutableString alloc] initWithString:filename];
 	[file replaceOccurrencesOfString:@"/" withString:@"_" options:NSCaseInsensitiveSearch range:NSMakeRange(0, [filename length])];
+	[file replaceOccurrencesOfString:@":" withString:@"_" options:NSCaseInsensitiveSearch range:NSMakeRange(0, [filename length])];
 	NSString *f = [NSString stringWithString:file];
 	[file release];
 	
@@ -142,20 +141,36 @@
 	else
 		[self setActiveObject:OBJECT_OTHER];
 	[deleteFromActiveObjectButton setEnabled:[self canDelete]];
+	
+	if (aResponder == njbTable)
+	{
+		[self showDrawerInfo];
+	}
+	// used to be [aResponder class] == [NSSearchField class]
+	else if (aResponder == njbSearchField)
+	{
+		[drawerController disableAll];
+	}
 }
 
 /* can we copy a file to the NJB?
  */
 - (BOOL)canCopyToNJB
 {
-	return ([browser path] != nil && [[browser path] length] > 0);
+	if (![theNJB isConnected])
+		return NO;
+	else
+		return ([browser path] != nil && [[browser path] length] > 0);
 }
 
 /* can we copy a file from the NJB?
  */
 - (BOOL)canCopyFromNJB
 {
-	return ([njbTable selectedRow] != -1);
+	if (![theNJB isConnected])
+		return NO;
+	else
+		return ([njbTable selectedRow] != -1);
 }
 
 /* called when we disconnect
@@ -224,17 +239,19 @@
  */
 - (IBAction)copyFromNJB:(id)sender
 {
-	NSEnumerator *trackEnumerator = [njbTable selectedRowEnumerator];
-	NSNumber *index;
-	while (index = [trackEnumerator nextObject])
+	NSArray *selectedObjects;
+	selectedObjects = [arrayController selectedObjects];
+	NSEnumerator *trackEnumerator = [selectedObjects objectEnumerator];
+	MyItem *item;
+	while (item = [trackEnumerator nextObject])
 	{
-		[self copyFileFromNJB:[index intValue]];
+		[self copyFileFromNJB:item];
 	}
 }
 
 /* actually copy the file, implemented by superclass
  */ 
-- (void)copyFileFromNJB:(int)index{}
+- (void)copyFileFromNJB:(MyItem *)item{}
 
 /* delete a file locally
  * will delete all files in a directory but not directories or recursively
@@ -257,7 +274,6 @@
 	NSString *currentFile;
 	while (currentFile = [enumerator nextObject])
 	{
-		NSLog(@"Removing file: %@", currentFile);
 		if (![fileManager removeFileAtPath:currentFile handler:nil])
 		{
 			NSRunAlertPanel(@"File Deletion", [NSString stringWithFormat:@"There was an error deleting file %@.", currentFile], @"OK", nil, nil);
@@ -266,6 +282,7 @@
 	}
 	[fileManager release];
 	[browser reloadData];
+	[self enableTransfers];
 }
 
 /* delete all the selected items off the NJB, after warning
@@ -276,27 +293,20 @@
 	if (result == NSAlertDefaultReturn)
 		return;
 	
-	/* build up an array of items to be deleted
+	/* get array of items to be deleted
 	 * we can't pass indexes as they aren't valid
 	 * after items have been deleted
 	 */
-	NSEnumerator *enumerator = [njbTable selectedRowEnumerator];
-	NSNumber *index;
-	NSMutableArray *selectedItems = [[NSMutableArray alloc] initWithCapacity:[njbTable numberOfSelectedRows]];
-	while (index = [enumerator nextObject])
-	{
-	  [selectedItems addObject:[[self itemArrayDisplaying] objectAtIndex:[index intValue]]];
-	}
-	enumerator = [selectedItems objectEnumerator];
+	NSArray *selectedObjects = [arrayController selectedObjects];
+	NSEnumerator *enumerator = [selectedObjects objectEnumerator];
 	MyItem *currentItem;
 	while (currentItem = [enumerator nextObject])
 	{
 		[self deleteFromNJB:currentItem];
 	}
-	[selectedItems release];
 }
 
-/* delete an off the NJB, to be implemented by superclass
+/* delete an item off the NJB, to be implemented by superclass
  */
 - (void)deleteFromNJB:(MyItem *)item{}
 
@@ -304,7 +314,16 @@
  */
 - (void)tableViewSelectionDidChange:(NSNotification *)aNotification
 {
+	// we might be called from code changing the selection
+	if (activeObject != OBJECT_NJB_TABLE && activeObject != OBJECT_FILE_BROWSER)
+		return;
+
+//	if ([njbTable numberOfSelectedRows] > 0)
 	[self enableTransfers];
+	
+	[self showDrawerInfo];
+	
+	[super tableViewSelectionDidChange:aNotification];
 }
 
 /* clicked on a local track/file so we can transfer
@@ -339,14 +358,41 @@
 	[self menuDelete];
 }
 
-/* adds the item to fullItemArray and calls
-* searchNJBTable. To be run when new item added to
-* list.
-*/
-- (void)addToFullItemArray:(MyItem *)item
+/* adds the item to the arrays and sorts
+ * if necessary
+ */
+- (void)addToArray:(MyItem *)item
 {
-	[fullItemArray addObject:item];
-	[self searchNJBTable];
+	[arrayController addObject:item];
+	[self sortArrays];
+	[self showItemCounts];
+}
+
+- (void)removeFromArray:(MyItem *)item
+{
+	[arrayController removeObject:item];
+	[self showItemCounts];
+}
+
+- (void)activate
+{
+	[super activate];
+	if (activeObject == OBJECT_NJB_TABLE)
+	{
+		[self showDrawerInfo];
+	}
+}
+
+/* show the info for the selected file/track in the table in the drawer
+ * to be implemented by super class
+ */
+- (void)showDrawerInfo{}
+
+- (void)tableView:(NSTableView *)tableView didClickTableColumn:(NSTableColumn *)tableColumn
+{	
+	// disable and clear the drawer
+	[drawerController clearAll];
+	[drawerController disableAll];
 }
 
 @end
