@@ -7,12 +7,12 @@
 
 #import <Foundation/Foundation.h>
 #include "libnjb.h"
-#import "MyMutableArray.h"
 #import "Track.h"
 #import "DataFile.h"
 #import "StatusDisplayer.h"
 #import "NJBTransactionResult.h"
 #import "Playlist.h"
+#import "Directory.h"
 
 @interface NJB : NSObject {
 	IBOutlet StatusDisplayer *statusDisplayer;
@@ -22,6 +22,16 @@
 	BOOL connected;
 	unsigned long long totalDiskSpace;
 	unsigned long long freeDiskSpace;
+	NSMutableArray *cachedTrackList;
+	// this is set when we are getting the track list so
+	// we know the cache should not be used
+	BOOL downloadingTracks;
+	njb_t njbs[NJB_MAX_DEVICES];
+	
+	NSString *deviceString;
+	NSString *firmwareVersionString;
+	NSString *deviceIDString;
+	NSString *deviceVersionString;
 }
 
 - (NJBTransactionResult *)connect;
@@ -31,15 +41,16 @@
 - (statusTypes)status;
 - (void)setStatus:(statusTypes)newStatus;
 - (BOOL)isConnected;
-- (MyMutableArray *)tracks;
-- (MyMutableArray *)dataFiles;
-- (MyMutableArray *)playlists;
-- (NJBTransactionResult *)deleteTrack:(NSNumber *)trackID;
+- (NSMutableArray *)tracks;
+- (NSMutableArray *)cachedTrackList;
+- (Directory *)dataFiles;
+- (NSMutableArray *)playlists;
+- (NJBTransactionResult *)deleteTrack:(Track *)track;
 - (NJBTransactionResult *)uploadTrack:(Track *)track;
 - (NJBTransactionResult *)downloadTrack:(Track *)track;
-- (NJBTransactionResult *)changeTrackTag:(Track *)track;
+- (NJBTransactionResult *)changeTrackTagTo:(Track *)newTrack from:(Track *)oldTrack;
 - (NJBTransactionResult *)downloadFile:(DataFile *)dataFile;
-- (NJBTransactionResult *)uploadFile:(DataFile *)dataFile;
+- (NJBTransactionResult *)uploadFile:(DataFile *)dataFile toFolder:(NSString *)path;
 - (NJBTransactionResult *)deleteFile:(NSNumber *)fileID;
 - (unsigned long long)totalDiskSpace;
 - (unsigned long long)freeDiskSpace;
@@ -51,6 +62,18 @@
 - (NJBTransactionResult *)updatePlaylist:(Playlist *)playlist;
 - (NJBTransactionResult *)deletePlaylist:(Playlist *)playlist;
 - (NSString *)productName;
+- (NJBTransactionResult *)changeTrackTagTo:(Track *)newTrack from:(Track *)oldTrack;
+- (NJBTransactionResult *)createFolder:(Directory *)dir inDir:(NSString *)path;
+- (NSString *)deviceString;
+- (NSString *)firmwareVersionString;
+- (NSString *)deviceIDString;
+- (NSString *)deviceVersionString;
+- (void)storeDeviceString;
+- (void)storeFirmwareVersionString;
+- (void)storeDeviceIDString;
+- (void)storeDeviceVersionString;
+- (int)batteryLevel;
+- (NSString *)batteryStatus;
 
 int progress (u_int64_t sent, u_int64_t total, const char* buf, unsigned len, void *data);
 
