@@ -6,7 +6,6 @@
 //
 
 // todo: can we find out how many tracks there are before we download for progress bar & set capacity in NSMutableArray?
-// update libnjb to current cvs
 
 /* An obj-c wrapper for libnjb
  * most methods return an NJBTransactionResult
@@ -20,7 +19,10 @@
 #import "Track.h"
 #include "string.h"
 #include "njbusb.h"
+#include "njb_error.h"
 #import "defs.h"
+#include <sys/stat.h>
+#include <unistd.h>
 
 // declared so can be accessed from C function progress
 // set to be the outlet statusDisplayer in awakeFromNib
@@ -30,6 +32,9 @@ StatusDisplayer *statusDisplayerGlobal;
 @interface NJB (PrivateAPI)
 - (void)updateDiskSpace;
 - (njb_songid_t *)songidStructFromTrack:(Track *)track;
+- (NSString *)njbErrorString;
+- (NSString *)rateString:(double) rate;
+- (NSString *)uniqueFilename:(NSString *)path;
 @end
 
 @implementation NJB
@@ -40,6 +45,7 @@ StatusDisplayer *statusDisplayerGlobal;
 {
 	if (self = [super init])
 	{
+		njb = NULL;
 		[self setDebug:0];
 	}
 	return self;
@@ -48,6 +54,13 @@ StatusDisplayer *statusDisplayerGlobal;
 - (void)dealloc
 {
 	[statusDisplayerGlobal release];
+	[cachedTrackList release];
+	
+	[deviceString release];
+	[firmwareVersionString release];
+	[deviceIDString release];
+	[deviceVersionString release];
+	
 	[super dealloc];
 }
 
@@ -75,6 +88,26 @@ StatusDisplayer *statusDisplayerGlobal;
 	return connected;
 }
 
+- (NSString *)deviceString
+{
+	return deviceString;
+}
+
+- (NSString *)firmwareVersionString
+{
+	return firmwareVersionString;
+}
+
+- (NSString *)deviceIDString
+{
+	return deviceIDString;
+}
+
+- (NSString *)deviceVersionString
+{
+	return deviceVersionString;
+}
+
 /**********************/
 
 - (void)awakeFromNib
@@ -82,11 +115,24 @@ StatusDisplayer *statusDisplayerGlobal;
 	statusDisplayerGlobal = [statusDisplayer retain];
 }
 
+- (NSString *)njbErrorString
+{
+	const char *sp;
+	NSMutableString *errorString = [[NSMutableString alloc] init];
+	NJB_Error_Reset_Geterror(njb);
+  while ((sp = NJB_Error_Geterror(njb)) != NULL)
+	{
+		[errorString appendString:[NSString stringWithFormat:@"%s, ", sp]];
+  }
+  njb_error_clear(njb);
+	
+	return [errorString autorelease];
+}
+
 /* connect to the NJB
  */
 - (NJBTransactionResult *)connect
-{
-	njb_t njbs[NJB_MAX_DEVICES];
+{	
 	int n;
 	
 	if (debug)
@@ -95,32 +141,35 @@ StatusDisplayer *statusDisplayerGlobal;
 	if (NJB_Discover(njbs, 0, &n) == -1)
 	{
 		fprintf(stderr, "could not locate any jukeboxes\n");
-		return [[[NJBTransactionResult alloc] initWithSuccess:NO] autorelease];
+		return [[[NJBTransactionResult alloc] initWithSuccess:NO resultString:@"Could not discover any jukeboxes"] autorelease];
 	}
 	
 	if (n == 0)
 	{
 		[self setStatus:STATUS_NO_NJB];
 		connected = NO;
-		return [[[NJBTransactionResult alloc] initWithSuccess:NO] autorelease];
+		return [[[NJBTransactionResult alloc] initWithSuccess:NO resultString:@"Could not locate any jukeboxes"] autorelease];
 	}
 	
 	njb = &njbs[0];
+	
 	if (NJB_Open(njb) == -1)
 	{
-		NJB_Error_Dump(njb, stderr);
+		NSString *error = [self njbErrorString];
+		NSLog(error);
 		[self setStatus:STATUS_COULD_NOT_OPEN];
 		connected = NO;
-		return [[[NJBTransactionResult alloc] initWithSuccess:NO] autorelease];
+		return [[[NJBTransactionResult alloc] initWithSuccess:NO resultString:error] autorelease];
 	}
 	
 	if (NJB_Capture(njb) == -1)
 	{
-		NJB_Error_Dump(njb, stderr);
+		NSString *error = [self njbErrorString];
+		NSLog(error);
 		[self setStatus:STATUS_COULD_NOT_CAPTURE];
 		[self disconnect];
 		connected = NO;
-		return [[[NJBTransactionResult alloc] initWithSuccess:NO] autorelease];
+		return [[[NJBTransactionResult alloc] initWithSuccess:NO resultString:error] autorelease];
 	}
 	
 	[self setStatus:STATUS_CONNECTED];
@@ -131,9 +180,18 @@ StatusDisplayer *statusDisplayerGlobal;
 	// enable UTF8
 	NJB_Set_Unicode(NJB_UC_UTF8);
 	
-	// send notification
-	[statusDisplayer postNotificationName:NOTIFICATION_CONNECTED object:self];
+	// check we have reset the cache, should have been done on disconnect
+	[cachedTrackList release];
+	cachedTrackList = nil;
 	
+  [self storeDeviceString];
+	[self storeFirmwareVersionString];
+	[self storeDeviceIDString];
+	[self storeDeviceVersionString];
+
+	// send notification, make sure this is last so all variables are set above
+	[statusDisplayer postNotificationName:NOTIFICATION_CONNECTED object:self];
+		
 	return [[[NJBTransactionResult alloc] initWithSuccess:YES] autorelease];
 }
 
@@ -143,13 +201,15 @@ StatusDisplayer *statusDisplayerGlobal;
 {
 	if ([self isConnected])
 	{
-		NSLog(@"Releasing...");
 		NJB_Release(njb);
-		NSLog(@"Disconnecting...");
 		NJB_Close(njb);
 		connected = NO;
 		[self setStatus:STATUS_DISCONNECTED];
 		
+		// lose the cache
+		[cachedTrackList release];
+		cachedTrackList = nil;
+		
 		// send notification
 		[statusDisplayer postNotificationName:NOTIFICATION_DISCONNECTED object:self];
 	}
@@ -157,7 +217,13 @@ StatusDisplayer *statusDisplayerGlobal;
 
 - (NSString *)productName
 {
-	return [NSString stringWithCString:njb->njbid->productName];
+	if (njb == NULL)
+		return nil;
+	const char *name = NJB_Get_Device_Name(njb, 1);
+	if (name != NULL)
+		return [NSString stringWithCString:name];
+	else
+		return nil;
 }
 
 - (NSString *)ownerString
@@ -170,12 +236,18 @@ StatusDisplayer *statusDisplayerGlobal;
 	return [nsOwnerString autorelease];
 }
 
-- (MyMutableArray *)tracks
+- (NSMutableArray *)tracks
 {
 	if (![self isConnected])
 		return nil;
-	MyMutableArray *tracks = [[MyMutableArray alloc] init];
-	NJB_Reset_Get_Track_Tag(njb);
+	if (cachedTrackList)
+	{
+		return [self cachedTrackList];
+	}
+	
+	downloadingTracks = YES;
+	cachedTrackList = [[NSMutableArray alloc] init];
+	NJB_Reset_Get_Track_Tag(njb);	
 	njb_songid_t *songtag;
 	njb_songid_frame_t *frame;
 	while ((songtag = NJB_Get_Track_Tag(njb))) {
@@ -222,7 +294,7 @@ StatusDisplayer *statusDisplayerGlobal;
 			else if (frame->type == NJB_TYPE_UINT32)
 				[track setTrackNumber:frame->data.u_int32_val];
 			// in case it's a string
-			else if (frame->type == NJB_TYPE_STRING)
+			else if (frame->type == NJB_TYPE_STRING && frame->data.strval != NULL)
 			{
 				NSString *trackNumber = [NSString stringWithCString:frame->data.strval];
 				[track setTrackNumber:(unsigned)[trackNumber intValue]];
@@ -231,21 +303,26 @@ StatusDisplayer *statusDisplayerGlobal;
 				NSLog(@"type not expected for FR_TRACK field %d", frame->type);
 		}
 		frame = NJB_Songid_Findframe(songtag, FR_CODEC);
-		if (frame != NULL && frame->data.strval == NULL)
-			[track setCodec:CODEC_UNDEFINED];
-		else
+		if (frame != NULL)
 		{
-			if (strcmp(frame->data.strval, NJB_CODEC_MP3) == 0)
-				[track setCodec:CODEC_MP3];
-			else if (strcmp(frame->data.strval, NJB_CODEC_WMA) == 0)
-				[track setCodec:CODEC_WMA];
-			else if (strcmp(frame->data.strval, NJB_CODEC_WAV) == 0)
-				[track setCodec:CODEC_WAV];
-			else
+			if (frame->data.strval == NULL)
 				[track setCodec:CODEC_UNDEFINED];
+			else
+			{
+				if (strcmp(frame->data.strval, NJB_CODEC_MP3) == 0)
+					[track setCodec:CODEC_MP3];
+				else if (strcmp(frame->data.strval, NJB_CODEC_WMA) == 0)
+					[track setCodec:CODEC_WMA];
+				else if (strcmp(frame->data.strval, NJB_CODEC_WAV) == 0)
+					[track setCodec:CODEC_WAV];
+				else if (strcmp(frame->data.strval, NJB_CODEC_AA) == 0)
+					[track setCodec:CODEC_AA];
+				else
+					[track setCodec:CODEC_UNDEFINED];
+			}
 		}
 		
-		[track setTrackID:songtag->trid];
+		[track setItemID:songtag->trid];
 		
 		frame = NJB_Songid_Findframe(songtag, FR_YEAR);
 		if (frame != NULL)
@@ -255,7 +332,7 @@ StatusDisplayer *statusDisplayerGlobal;
 			else if (frame->type == NJB_TYPE_UINT32)
 				[track setYear:frame->data.u_int32_val];
 			// strings on NJB1
-			else if (frame->type == NJB_TYPE_STRING)
+			else if (frame->type == NJB_TYPE_STRING && frame->data.strval != NULL)
 			{
 				NSString *year = [NSString stringWithCString:frame->data.strval];
 				[track setYear:(unsigned)[year intValue]];
@@ -266,12 +343,13 @@ StatusDisplayer *statusDisplayerGlobal;
 		else
 			[track setYear:0];
 		
-		[tracks addObject:track];
+		[cachedTrackList addObject:track];
 		[track release];
 		
 		NJB_Songid_Destroy(songtag);
 	}
-	return [tracks autorelease];
+	downloadingTracks = NO;
+	return [self cachedTrackList];
 }
 
 /* uploads the track and sets the track id
@@ -280,40 +358,52 @@ StatusDisplayer *statusDisplayerGlobal;
 {
 	NSDate *date = [NSDate date];
 	if (![self isConnected])
-		return [[[NJBTransactionResult alloc] initWithSuccess:NO] autorelease];
-	NSLog(@"Uploading track: %@", [track fullPath]);
+		return [[[NJBTransactionResult alloc] initWithSuccess:NO resultString:@"Not connected"] autorelease];
 	[self setStatus:STATUS_UPLOADING_TRACK];
-	int trackid;
+	unsigned int trackid;
 	njb_songid_t *songid = [self songidStructFromTrack:track];
 	if (NJB_Send_Track(njb, [[track fullPath] fileSystemRepresentation], songid, progress, NULL, &trackid) == -1 ) {
-		NJB_Error_Dump(njb, stderr);
+		NSString *error = [self njbErrorString];
+		NSLog(error);
 		NJB_Songid_Destroy(songid);
-		return [[[NJBTransactionResult alloc] initWithSuccess:NO] autorelease];
+		return [[[NJBTransactionResult alloc] initWithSuccess:NO resultString:error] autorelease];
 	} else {
 		[self updateDiskSpace];
-		[track setTrackID:trackid];
+		[track setItemID:trackid];
 		
 		double time = -[date timeIntervalSinceNow];
-		double rate = (double)[track filesize] / time;
-		NSLog(@"Took: %f seconds, speed %f Mbps", time, rate * 8.0 / (1024.0 * 1024.0));
+		// rate in Mbps
+		double rate = ((double)[track filesize] / time) * 8.0 / (1024.0 * 1024.0);
 		
 		NJB_Songid_Destroy(songid);
-		return [[[NJBTransactionResult alloc] initWithSuccess:YES] autorelease];
+		// update my cache
+		[cachedTrackList addObject:track];
+		// tell others the track list has changed
+		[statusDisplayer postNotificationName:NOTIFICATION_TRACK_LIST_MODIFIED object:self];
+		
+		return [[[NJBTransactionResult alloc] initWithSuccess:YES
+																						 resultString:[NSString stringWithFormat:@"Speed %@ Mbps", [self rateString:rate]]] autorelease];
 	}
 }
 
-- (NJBTransactionResult *)deleteTrack:(NSNumber *)trackID
+- (NJBTransactionResult *)deleteTrack:(Track *)track
 {
 	if (![self isConnected])
 		return [[[NJBTransactionResult alloc] initWithSuccess:NO] autorelease];
-	if (NJB_Delete_Track(njb, [trackID unsignedIntValue]) == -1)
+	if (NJB_Delete_Track(njb, [track itemID]) == -1)
 	{
-		NJB_Error_Dump(njb, stderr);
-		return [[[NJBTransactionResult alloc] initWithSuccess:NO] autorelease];
+		NSString *error = [self njbErrorString];
+		NSLog(error);
+		return [[[NJBTransactionResult alloc] initWithSuccess:NO resultString:error] autorelease];
 	}
 	else
 	{
 		[self updateDiskSpace];
+		// update my cache
+		[cachedTrackList removeObject:track];
+		// tell others the track list has changed
+		[statusDisplayer postNotificationName:NOTIFICATION_TRACK_LIST_MODIFIED object:self];
+		
 		return [[[NJBTransactionResult alloc] initWithSuccess:YES] autorelease];
 	}
 }
@@ -324,8 +414,9 @@ StatusDisplayer *statusDisplayerGlobal;
 		return [[[NJBTransactionResult alloc] initWithSuccess:NO] autorelease];
 	if (NJB_Delete_Datafile(njb, [fileID unsignedIntValue]) == -1)
 	{
-		NJB_Error_Dump(njb, stderr);
-		return [[[NJBTransactionResult alloc] initWithSuccess:NO] autorelease];
+		NSString *error = [self njbErrorString];
+		NSLog(error);
+		return [[[NJBTransactionResult alloc] initWithSuccess:NO resultString:error] autorelease];
 	}
 	else
 	{
@@ -336,99 +427,174 @@ StatusDisplayer *statusDisplayerGlobal;
 
 - (NJBTransactionResult *)downloadTrack:(Track *)track
 {
+	// check to see we're not going to overwrite anything
+	NSString *fullPath = [track fullPath];
+	fullPath = [self uniqueFilename:fullPath];
+	[track setFullPath:fullPath];
+	
 	NSDate *date = [NSDate date];
-	if (NJB_Get_Track(njb, [track trackID], [track filesize], [[track fullPath] fileSystemRepresentation], progress, NULL) == -1)
+	if (NJB_Get_Track(njb, [track itemID], [track filesize], [[track fullPath] fileSystemRepresentation], progress, NULL) == -1)
 	{
-		NJB_Error_Dump(njb, stderr);
-		return [[[NJBTransactionResult alloc] initWithSuccess:NO] autorelease];
+		NSString *error = [self njbErrorString];
+		NSLog(error);
+		return [[[NJBTransactionResult alloc] initWithSuccess:NO resultString:error] autorelease];
 	}
 	double time = -[date timeIntervalSinceNow];
-	double rate = (double)[track filesize] / time;
-	NSLog(@"Took: %f seconds, speed %f Mbps", time, rate * 8.0 / (1024.0 * 1024.0));
+	// rate in Mbps
+	double rate = ((double)[track filesize] / time) * 8.0 / (1024.0 * 1024.0);
 	
-	return [[[NJBTransactionResult alloc] initWithSuccess:YES] autorelease];
+	return [[[NJBTransactionResult alloc] initWithSuccess:YES
+																					 resultString:[NSString stringWithFormat:@"Speed %@ Mbps", [self rateString:rate]]] autorelease];
 }
 
-// todo: does NJB_Replace_Track_Tag ever fail?
-- (NJBTransactionResult *)changeTrackTag:(Track *)track
+- (NSString *)uniqueFilename:(NSString *)path
 {
-	njb_songid_t *songid = [self songidStructFromTrack:track];
-	
-	if (NJB_Replace_Track_Tag(njb, [track trackID], songid) == -1)
+	struct stat sb;
+	while (stat([path fileSystemRepresentation], &sb) != -1)
 	{
-		NJB_Error_Dump(njb, stderr);
+		NSString *extension = [path pathExtension];
+		NSString *pathWithoutExtension = [path stringByDeletingPathExtension];
+		pathWithoutExtension = [pathWithoutExtension stringByAppendingString:@"_"];
+		path = [pathWithoutExtension stringByAppendingPathExtension:extension];
+	}
+	return path;
+}
+
+- (NJBTransactionResult *)changeTrackTagTo:(Track *)newTrack from:(Track *)oldTrack
+{
+	njb_songid_t *songid = [self songidStructFromTrack:newTrack];
+	
+	if (NJB_Replace_Track_Tag(njb, [newTrack itemID], songid) == -1)
+	{
+		NSString *error = [self njbErrorString];
+		NSLog(error);
 		NJB_Songid_Destroy(songid);
-		return [[[NJBTransactionResult alloc] initWithSuccess:NO] autorelease];
+		return [[[NJBTransactionResult alloc] initWithSuccess:NO resultString:error] autorelease];
 	}
 	else
 	{
 		[self updateDiskSpace];
 		NJB_Songid_Destroy(songid);
+		
+		// update my cache
+		[cachedTrackList replaceObject:oldTrack withObject:newTrack];
+		
+		// tell others the track list has changed
+		[statusDisplayer postNotificationName:NOTIFICATION_TRACK_LIST_MODIFIED object:self];
+		
 		return [[[NJBTransactionResult alloc] initWithSuccess:YES] autorelease];
 	}
 }
 
-- (MyMutableArray *)dataFiles
+- (Directory *)dataFiles
 {
 	if (![self isConnected])
 		return nil;
-	MyMutableArray *dataFiles = [[MyMutableArray alloc] init];
+	
+	Directory *baseDir = [[Directory alloc] init];
 	NJB_Reset_Get_Datafile_Tag(njb);
 	njb_datafile_t *filetag;
-	while ((filetag = NJB_Get_Datafile_Tag (njb)))
+	while (filetag = NJB_Get_Datafile_Tag(njb))
 	{
-		DataFile *dataFile = [[DataFile alloc] init];
-		const char *str = filetag->filename;
-		if (str != NULL)
-			[dataFile setFilename:[NSString stringWithUTF8String:str]];
-		[dataFile setSize:filetag->filesize];
-//		[dataFile setTimestampSince1970:filetag->timestamp];
-		[dataFile setFileID:filetag->dfid];
+		NSString *filename = @"";
+		if (filetag->filename != NULL)
+			filename = [NSString stringWithCString:filetag->filename];
+		NSArray *pathArray = nil;
+		if (filetag->folder != NULL)
+		{
+			NSString *folder = [NSString stringWithCString:filetag->folder];
+			// this has \\ instead of /
+			pathArray = [folder componentsSeparatedByString:@"\\"];
+			// the / on the beginning and end gives us extra empty objects
+			NSMutableArray *newPathArray = [NSMutableArray arrayWithArray:pathArray];
+			if ([[newPathArray objectAtIndex:[newPathArray count] - 1] length] == 0)
+				[newPathArray removeLastObject];
+			if ([[newPathArray objectAtIndex:0] length] == 0)
+				[newPathArray removeObjectAtIndex:0];
+			pathArray = newPathArray;
+		}
+
+		if ([filename isEqualToString:@"."])
+		{
+			// this is an empty folder
+			NSMutableArray *parentPathArray = [NSMutableArray arrayWithArray:pathArray];
+			[parentPathArray removeLastObject];
+			
+			NSString *name = [pathArray objectAtIndex:[pathArray count] - 1];
+
+			Directory *parentDir = [baseDir itemWithPath:parentPathArray];
+			if (![parentDir containsItemWithName:name])
+			{
+				Directory *newDir = [[Directory alloc] initWithName:name];
+				[newDir setItemID:filetag->dfid];
+			
+				[baseDir addItem:newDir toDir:parentPathArray];
+			
+				[newDir release];
+			}
+		}
+		else
+		{
+			// this is a file
+			DataFile *dataFile = [[DataFile alloc] init];
+			[dataFile setFilename:filename];
+			[dataFile setSize:filetag->filesize];
+			// [dataFile setTimestampSince1970:filetag->timestamp];
+			[dataFile setItemID:filetag->dfid];
+			
+			[baseDir addItem:dataFile toDir:pathArray];
+			
+			[dataFile release];
+		}
 		
-		[dataFiles addObject:dataFile];
-		[dataFile release];
-		
-  	 NJB_Datafile_Destroy(filetag);
+		NJB_Datafile_Destroy(filetag);
 	}
-	return [dataFiles autorelease];
+	
+	return [baseDir autorelease];
 }
 
 - (NJBTransactionResult *)downloadFile:(DataFile *)dataFile
 {
+	// check to see we're not going to overwrite anything
+	NSString *fullPath = [dataFile fullPath];
+	fullPath = [self uniqueFilename:fullPath];
+	[dataFile setFullPath:fullPath];
+	
 	NSDate *date = [NSDate date];
-	if (NJB_Get_File(njb, [dataFile fileID], [dataFile size], [[dataFile fullPath] fileSystemRepresentation], progress, NULL) == -1)
+	if (NJB_Get_File(njb, [dataFile itemID], [dataFile size], [[dataFile fullPath] fileSystemRepresentation], progress, NULL) == -1)
 	{
-		NJB_Error_Dump(njb, stderr);
-		return [[[NJBTransactionResult alloc] initWithSuccess:NO] autorelease];
+		NSString *error = [self njbErrorString];
+		NSLog(error);
+		return [[[NJBTransactionResult alloc] initWithSuccess:NO resultString:error] autorelease];
 	}
 	double time = -[date timeIntervalSinceNow];
-	double rate = (double)[dataFile size] / time;
-	NSLog(@"Took: %f seconds, speed %f Mbps", time, rate * 8.0 / (1024.0 * 1024.0));
+	// rate in Mbps
+	double rate = ((double)[dataFile size] / time) * 8.0 / (1024.0 * 1024.0);
 		
-	return [[[NJBTransactionResult alloc] initWithSuccess:YES] autorelease];
+	return [[[NJBTransactionResult alloc] initWithSuccess:YES
+																					 resultString:[NSString stringWithFormat:@"Speed %@ Mbps", [self rateString:rate]]] autorelease];
 }
 
-- (NJBTransactionResult *)uploadFile:(DataFile *)dataFile
+- (NJBTransactionResult *)uploadFile:(DataFile *)dataFile toFolder:(NSString *)path
 {
 	if (![self isConnected])
 		return [[[NJBTransactionResult alloc] initWithSuccess:NO] autorelease];
-	NSLog(@"Uploading file: %@", [dataFile fullPath]);
 	NSDate *date = [NSDate date];
 	[self setStatus:STATUS_UPLOADING_FILE];
-	int fileid;
-	int NJB_Send_File (njb_t *njb, const char *path, const char *name, const char *folder,
-										 NJB_Xfer_Callback *callback, void *data, u_int32_t *fileid);
-	if (NJB_Send_File(njb, [[dataFile fullPath] fileSystemRepresentation], [[[dataFile fullPath] lastPathComponent] UTF8String], NULL, progress, NULL, &fileid) == -1 ) {
-		NJB_Error_Dump(njb, stderr);
-		return [[[NJBTransactionResult alloc] initWithSuccess:NO] autorelease];
+	unsigned int fileid;
+	if (NJB_Send_File(njb, [[dataFile fullPath] fileSystemRepresentation], [[[dataFile fullPath] lastPathComponent] UTF8String], [path UTF8String], progress, NULL, &fileid) == -1 ) {
+		NSString *error = [self njbErrorString];
+		NSLog(error);
+		return [[[NJBTransactionResult alloc] initWithSuccess:NO resultString:error] autorelease];
 	} else {
-		[dataFile setFileID:fileid];
+		[dataFile setItemID:fileid];
 		[self updateDiskSpace];
 		
 		double time = -[date timeIntervalSinceNow];
-		double rate = (double)[dataFile size] / time;
-		NSLog(@"Took: %f seconds, speed %f Mbps", time, rate * 8.0 / (1024.0 * 1024.0));
-		return [[[NJBTransactionResult alloc] initWithSuccess:YES] autorelease];
+		// rate in Mbps
+		double rate = ((double)[dataFile size] / time) * 8.0 / (1024.0 * 1024.0);
+		return [[[NJBTransactionResult alloc] initWithSuccess:YES
+																						 resultString:[NSString stringWithFormat:@"Speed %@ Mbps", [self rateString:rate]]] autorelease];
 	}
 }
 
@@ -469,18 +635,20 @@ StatusDisplayer *statusDisplayerGlobal;
 {
 	if (NJB_Set_Owner_String (njb, [owner UTF8String]) == -1)
 	{
-		NJB_Error_Dump(njb, stderr);
-		return [[[NJBTransactionResult alloc] initWithSuccess:NO] autorelease];
+		NSString *error = [self njbErrorString];
+		NSLog(error);
+		return [[[NJBTransactionResult alloc] initWithSuccess:NO resultString:error] autorelease];
 	}
 	return [[[NJBTransactionResult alloc] initWithSuccess:YES] autorelease];
 }
 
 - (NJBTransactionResult *)setBitmap:(NSString *)bitmapPath
 {
-	if (NJB_Set_Bitmap (njb, [bitmapPath UTF8String]) == -1)
+	if (NJB_Set_Bitmap(njb, [bitmapPath UTF8String]) == -1)
 	{
-		NJB_Error_Dump(njb, stderr);
-		return [[[NJBTransactionResult alloc] initWithSuccess:NO] autorelease];
+		NSString *error = [self njbErrorString];
+		NSLog(error);
+		return [[[NJBTransactionResult alloc] initWithSuccess:NO resultString:error] autorelease];
 	}
 	return [[[NJBTransactionResult alloc] initWithSuccess:YES] autorelease];
 }
@@ -502,18 +670,19 @@ StatusDisplayer *statusDisplayerGlobal;
 	
 	if (NJB_Set_Time(njb, &time) == -1)
 	{
-		NJB_Error_Dump(njb, stderr);
-		return [[[NJBTransactionResult alloc] initWithSuccess:NO] autorelease];
+		NSString *error = [self njbErrorString];
+		NSLog(error);
+		return [[[NJBTransactionResult alloc] initWithSuccess:NO resultString:error] autorelease];
 	}
 	return [[[NJBTransactionResult alloc] initWithSuccess:YES] autorelease];
 }
 
-- (MyMutableArray *)playlists
+- (NSMutableArray *)playlists
 {
 	if (![self isConnected])
 		return nil;
 	
-	MyMutableArray *playlists = [[MyMutableArray alloc] init];
+	NSMutableArray *playlists = [[NSMutableArray alloc] init];
 	NJB_Reset_Get_Playlist(njb);
 	njb_playlist_t *playlist;
 	while ((playlist = NJB_Get_Playlist(njb)))
@@ -530,7 +699,7 @@ StatusDisplayer *statusDisplayerGlobal;
 		while ((track = NJB_Playlist_Gettrack(playlist)))
 		{
 			Track *newTrack = [[Track alloc] init];
-			[newTrack setTrackID:track->trackid];
+			[newTrack setItemID:track->trackid];
 			[newPlaylist addTrack:newTrack];
 			[newTrack release];
 		}
@@ -548,8 +717,6 @@ StatusDisplayer *statusDisplayerGlobal;
 /* updates the playlist on the NJB
  * may add a new playlist even if playlist state is not new
  * so the playlist ID may change.
- * This is implemented by modifying libnjb (njb3_add_multiple_tracks_to_playlist)
- * for the NJB3 protocol
  */
 - (NJBTransactionResult *)updatePlaylist:(Playlist *)playlist
 {
@@ -569,7 +736,7 @@ StatusDisplayer *statusDisplayerGlobal;
 	Track *currentTrack;
 	while (currentTrack = [enumerator nextObject])
 	{
-		njb_playlist_track_t *newTrack = NJB_Playlist_Track_New([currentTrack trackID]);
+		njb_playlist_track_t *newTrack = NJB_Playlist_Track_New([currentTrack itemID]);
 		NJB_Playlist_Addtrack(pl, newTrack, NJB_PL_END);
 	}
 	pl->_state = [playlist state];
@@ -579,8 +746,9 @@ StatusDisplayer *statusDisplayerGlobal;
 	if (NJB_Update_Playlist(njb, pl) == -1)
 	{
 		NJB_Playlist_Destroy(pl);
-		NJB_Error_Dump(njb, stderr);
-		return [[[NJBTransactionResult alloc] initWithSuccess:NO] autorelease];
+		NSString *error = [self njbErrorString];
+		NSLog(error);
+		return [[[NJBTransactionResult alloc] initWithSuccess:NO resultString:error] autorelease];
 	}
 	[playlist lock];
 	// update the ID as this is changed when tracks added/removed as a new playlist is created
@@ -598,8 +766,9 @@ StatusDisplayer *statusDisplayerGlobal;
 	[playlist unlock];
 	if (NJB_Delete_Playlist(njb, plid) == -1)
 	{
-		NJB_Error_Dump(njb, stderr);
-		return [[[NJBTransactionResult alloc] initWithSuccess:NO] autorelease];
+		NSString *error = [self njbErrorString];
+		NSLog(error);
+		return [[[NJBTransactionResult alloc] initWithSuccess:NO resultString:error] autorelease];
 	}
 	return [[[NJBTransactionResult alloc] initWithSuccess:YES] autorelease];
 }
@@ -667,4 +836,115 @@ int progress(u_int64_t sent, u_int64_t total, const char* buf, unsigned len, voi
 	return songid;
 }
 
+- (NSMutableArray *)cachedTrackList
+{
+	if (cachedTrackList && !downloadingTracks)
+	{
+		NSMutableArray *tracks = [[NSMutableArray alloc] initWithArray:cachedTrackList];
+		return [tracks autorelease];
+	}
+	else
+		return nil;
+}
+
+/* returns a string rounded to 1dp for
+ * the rate given
+ */
+- (NSString *)rateString:(double) rate
+{
+	int rate10 = rate * 10;
+	if (rate - (double)rate10/10.0 >= 0.05)
+		rate10++;
+	// the two digits
+	int rateA = rate10/10;
+	int rateB = rate10-rateA*10;
+	return [NSString stringWithFormat:@"%d.%d", rateA, rateB];
+}
+
+- (NJBTransactionResult *)createFolder:(Directory *)dir inDir:(NSString *)path
+{
+	unsigned int folderID;
+	if (NJB_Create_Folder(njb, [[NSString stringWithFormat:@"%@%@", path, [dir name]] UTF8String], &folderID) == -1)
+	{
+		NSString *error = [self njbErrorString];
+		NSLog(error);
+		return [[[NJBTransactionResult alloc] initWithSuccess:NO resultString:error] autorelease];
+	}
+	[dir setItemID:folderID];
+	return [[[NJBTransactionResult alloc] initWithSuccess:YES] autorelease];
+}
+
+- (void)storeDeviceString
+{
+	const char *name = NJB_Get_Device_Name(njb, 0);
+	if (name != NULL)
+		deviceString = [[NSString alloc] initWithCString:name];
+	else
+		deviceString = nil;
+}
+
+- (void)storeFirmwareVersionString
+{
+	u_int8_t major, minor, release;
+		
+	int ret = NJB_Get_Firmware_Revision(njb, &major, &minor, &release);
+	if (ret == 0)
+		firmwareVersionString = [[NSString alloc] initWithString:[NSString stringWithFormat:@"%u.%u.%u", major, minor, release]];
+	else
+		deviceVersionString = nil;
+}
+
+- (void)storeDeviceIDString
+{	
+	NSMutableString *idString = [[NSMutableString alloc] initWithCapacity:32];
+	int j;
+	
+	u_int8_t sdmiid[16];
+	int result;
+	result = NJB_Get_SDMI_ID(njb, &(sdmiid[0]));
+	if (result != 0)
+		idString = @"";
+	
+	for (j = 0; j < 16; j++)
+	{
+		if (sdmiid[j] > 15)
+			[idString appendString:[NSString stringWithFormat:@"%X", sdmiid[j]]];
+		else
+			[idString appendString:[NSString stringWithFormat:@"0%X", sdmiid[j]]];
+	}
+	
+	deviceIDString = [[NSString alloc] initWithString:[NSString stringWithString:idString]];
+	[idString release];
+}
+
+- (void)storeDeviceVersionString
+{
+	u_int8_t major, minor, release;
+		
+	int ret = NJB_Get_Hardware_Revision(njb, &major, &minor, &release);
+	if (ret == 0)
+		deviceVersionString = [[NSString alloc] initWithString:[NSString stringWithFormat:@"%u.%u.%u", major, minor, release]];
+	else
+		deviceVersionString = nil;
+}
+
+- (int)batteryLevel
+{
+	if (connected == NO)
+		return 0;
+	return NJB_Get_Battery_Level(njb);
+}
+
+- (NSString *)batteryStatus
+{
+	BOOL charging = (NJB_Get_Battery_Charging(njb) == 1);
+	BOOL powerConnected = (NJB_Get_Auxpower(njb) == 1);
+	
+	if (charging)
+		return @"Charging";
+	if (powerConnected)
+		return @"Battery charged or not present";
+	return @"Running off battery";
+}
+
 @end
