@@ -22,32 +22,6 @@
 #define COLUMN_FILESIZE @"Filesize"
 #define COLUMN_YEAR @"Year"
 
-// add - (Track *)trackWithID:(unsigned)trackID to MyMutableArray
-@interface MyMutableArray (trackWithID)
-- (Track *)trackWithID:(unsigned)trackID;
-@end
-
-// adds to MyMutableArray a method to return the track with ID trackID
-@implementation MyMutableArray (trackWithID)
-
-/* finds a track which has ID trackID
- * assumes all IDs are unique so returns first item only
- * returns nil if none found
- */
-- (Track *)trackWithID:(unsigned)trackID
-{
-	NSEnumerator *enumerator = [objects objectEnumerator];
-	Track *currentTrack;
-	while (currentTrack = [enumerator nextObject])
-	{
-		if ([currentTrack trackID] == trackID)
-			return currentTrack;
-	}
-	return nil;
-}
-
-@end
-
 // declare the private methods
 @interface PlaylistsTab (PrivateAPI)
 - (void)fillPlaylistTrackInfo;
@@ -60,95 +34,37 @@
 
 // init/dealloc methods
 
-- (id)init
-{
-	if (self = [super init])
-	{
-		trackListIsUpToDate = NO;
-	}
-	return self;
-}
-
 - (void)dealloc
 {
 	[playlists release];
 	[super dealloc];
 }
 
-// accessor methods
-
-- (BOOL)trackListIsUpToDate
-{
-	return trackListIsUpToDate;
-}
-
 /**********************/
 
 - (void)onConnectAndActive
 {
 	[super onConnectAndActive];
-	if ([musicTab trackListIsUpToDate] && [musicTab fullItemArray])
-	{
-		// musicTab has already got it
-		[fullItemArray release];
-		fullItemArray = [[MyMutableArray alloc] initWithMyMutableArray:[musicTab fullItemArray]];
-		[self clearSortingArrows];
-		[self searchNJBTable];
-		trackListIsUpToDate = YES;
-	}
-	else
-	{
+	if (!trackListUpToDate)
 		[self loadTracks];
-		trackListIsUpToDate = YES;
-	}
 	[self loadPlaylists];
 }
 
-- (void)awakeFromNib
+- (void)activate
 {
-	[super awakeFromNib];
-	
-	[njbTable setAutosaveName:PREF_KEY_PLAYLISTS_TAB_TABLE];
-	[njbTable setAutosaveTableColumns:YES];
-	[playlistsTable setAutosaveName:PREF_KEY_PLAYLISTS_TAB_PLAYLISTS_TABLE];
-	[playlistsTable setAutosaveTableColumns:YES];
-	[playlistsTrackTable setAutosaveName:PREF_KEY_PLAYLISTS_TAB_PLAYLISTS_TRACK_TABLE];
-	[playlistsTrackTable setAutosaveTableColumns:YES];
+	// this must be called first to run onConnectAndActive so we can download the tracks there
+	[super activate];
+	if (!trackListUpToDate)
+	{
+		[self loadTracks];
+	}
 }
 
 - (id)tableView:(NSTableView *)aTableView
    objectValueForTableColumn:(NSTableColumn *)aTableColumn
 						row:(int)rowIndex
 {	
-	if (aTableView == njbTable)
-	{
-		NSString *ident = [aTableColumn identifier];
-		Track *track = (Track *)[[self itemArrayDisplaying] objectAtIndex:rowIndex];
-		if ([ident isEqual:COLUMN_TITLE])
-			return [track title];
-		else if ([ident isEqual:COLUMN_ALBUM])
-			return [track album];
-		else if ([ident isEqual:COLUMN_ARTIST])
-			return [track artist];
-		else if ([ident isEqual:COLUMN_GENRE])
-			return [track genre];
-		else if ([ident isEqual:COLUMN_LENGTH])
-			return [track lengthString];
-		else if ([ident isEqual:COLUMN_TRACK_NO])
-			return [NSString stringWithFormat:@"%d", [track trackNumber]];
-		else if ([ident isEqual:COLUMN_CODEC])
-			return [track codecString];
-		else if ([ident isEqual:COLUMN_FILESIZE])
-			return [track filesizeString];
-		else if ([ident isEqual:COLUMN_YEAR])
-			return [NSString stringWithFormat:@"%d", [track year]];
-		else
-		{
-			NSLog(@"Invalid column tag in PlaylistsTab for njbTable");
-			return nil;
-		}
-	}
-	else if (aTableView == playlistsTable)
+	if (aTableView == playlistsTable)
 	{
 		Playlist *playlist = (Playlist *)[playlists objectAtIndex:rowIndex];
 	  return [playlist name];
@@ -172,15 +88,13 @@
 		else
 			return nil;
 	}
-	NSLog(@"Unknown table %@ in PlaylistsTab tableView:objectValueForTableColumn:row:!", aTableView);
+	//NSLog(@"Unknown table %@ in PlaylistsTab tableView:objectValueForTableColumn:row:!", aTableView);
 	return nil;
 }
 
 - (int)numberOfRowsInTableView:(NSTableView *)aTableView
 {
-	if (aTableView == njbTable)
-		return [super numberOfRowsInTableView:aTableView];
-	else if (aTableView == playlistsTable)
+	if (aTableView == playlistsTable)
 		return [playlists count];
 	else if (aTableView == playlistsTrackTable)
 	{
@@ -190,80 +104,46 @@
 			return 0;
 	}
 	// may get here if any of the table pointers above haven't been set yet
-	NSLog(@"Unknown table %@ in PlaylistsTab numberOfRowsInTableView:", aTableView);
+	//NSLog(@"Unknown table %@ in PlaylistsTab numberOfRowsInTableView:", aTableView);
 	return 0;
 }
 
-- (SEL)sortSelectorForColumn:(NSString *)columnID
-{
-	if ([columnID isEqual:COLUMN_TITLE])
-		return [Track compareByTitleSelector];
-	else if ([columnID isEqual:COLUMN_ARTIST])
-		return [Track compareByArtistSelector];
-	else if ([columnID isEqual:COLUMN_ALBUM])
-		return [Track compareByAlbumSelector];
-	else if ([columnID isEqual:COLUMN_GENRE])
-		return [Track compareByGenreSelector];
-	else if ([columnID isEqual:COLUMN_LENGTH])
-		return [Track compareByLengthSelector];
-	else if ([columnID isEqual:COLUMN_TRACK_NO])
-		return [Track compareByTrackNumberSelector];
-	else if ([columnID isEqual:COLUMN_CODEC])
-		return [Track compareByCodecSelector];
-	else if ([columnID isEqual:COLUMN_FILESIZE])
-		return [Track compareByFilesizeSelector];
-	else if ([columnID isEqual:COLUMN_YEAR])
-		return [Track compareByYearSelector];
-	else
-	{
-		NSLog(@"Invalid column %@ in sortSelectorForColumn!", columnID);
-		return nil;
-	}
-}
-
-- (void)searchNJBTable
-{	
-	NSString *searchString = [njbSearchField stringValue];
-	// release old track list
-	[truncatedItemArray release];
-	
-	if ([searchString length] == 0)
-	{
-		truncatedItemArray = nil;
-		[njbTable reloadData];
-		return;
-	}
-	
-	truncatedItemArray = [fullItemArray searchFor:searchString];
-	[truncatedItemArray retain];
-	
-	[njbTable reloadData];
-}
-
 /* add downloadTrackList to the njb queue
  */
 - (void)loadTracks
 {
-	NJBQueueItem *clearSortingArrows = [[NJBQueueItem alloc] initWithTarget:self
-																														 withSelector:@selector(clearSortingArrows)
-																															 withObject:nil
-																											withRunInMainThread:YES];
-	[clearSortingArrows setDisplayStatus:NO];
+	if (trackListUpToDate)
+	{
+		NSLog(@"loadTracks called in Playlists tab when track list up to date");
+		return;
+	}
+	NSMutableArray *cachedTrackList = [theNJB cachedTrackList];
+	if (cachedTrackList != nil)
+	{
+		[tempArray release];
+		tempArray = cachedTrackList;
+		[tempArray retain];
+		[self showNewArray];
+		
+		trackListUpToDate = YES;
+		return;
+	}
 	
 	NJBQueueItem *getTracks = [[NJBQueueItem alloc] initWithTarget:self withSelector:@selector(downloadTrackList)];
 	[getTracks setStatus:STATUS_DOWNLOADING_TRACK_LIST];
 	
 	NJBQueueItem *updateTable = [[NJBQueueItem alloc] initWithTarget:self
-																											withSelector:@selector(searchNJBTable)
+																											withSelector:@selector(showNewArray)
 																												withObject:nil
 																							 withRunInMainThread:YES];
 	[updateTable setDisplayStatus:NO];
 	
 	NSString *description = @"Getting track list";
-	[self addToQueue:getTracks withSubItems:[NSArray arrayWithObjects:clearSortingArrows, updateTable, nil] withDescription:description];
+	[self addToQueue:getTracks withSubItems:[NSArray arrayWithObjects:updateTable, nil] withDescription:description];
 	[getTracks release];
-	[clearSortingArrows release];
 	[updateTable release];
+	// it's not actually yet, but will be in a while, don't want to go copying anything
+	trackListUpToDate = YES;
 }
 
 /* this is called from the queue consumer 
@@ -272,13 +152,13 @@
  */
 - (NJBTransactionResult *)downloadTrackList
 {
-	NSLog(@"Getting tracks");
-	[fullItemArray release];
-	fullItemArray = [theNJB tracks];
+	[tempArray release];
+	tempArray = [theNJB tracks];
+	[tempArray retain];
 	
-	NJBTransactionResult *result = [[NJBTransactionResult alloc] initWithSuccess:(fullItemArray != nil)];
+	NJBTransactionResult *result = [[NJBTransactionResult alloc] initWithSuccess:(tempArray != nil)];
 	
-	[fullItemArray retain];
+	trackListUpToDate = YES;
 	
 	return [result autorelease];
 }
@@ -320,7 +200,6 @@
  */
 - (NJBTransactionResult *)downloadPlaylists
 {
-	NSLog(@"Getting playlists");
 	[playlists release];
 	
 	playlists = [theNJB playlists];
@@ -341,7 +220,7 @@
 	Playlist *currentPlaylist;
 	while (currentPlaylist = [enumerator nextObject])
 	{
-		[currentPlaylist fillTrackInfoFromList:fullItemArray];
+		[currentPlaylist fillTrackInfoFromList:[arrayController content]];
 	}
 }
 
@@ -352,21 +231,6 @@
 	[newPlaylistButton setEnabled:YES];
 }
 
-/* called by MusicTab when it has modified the track list
- */
-- (void)updateTracks
-{
-	if (fullItemArray != nil)
-	{
-		// this doesn't resort, could be annoying if someone is browsing in playlists tab
-		// and it gets resorted
-		[self clearSortingArrows];
-		[fullItemArray release];
-		fullItemArray = [[MyMutableArray alloc] initWithMyMutableArray:[musicTab fullItemArray]];
-		[self searchNJBTable];
-	}
-}
-
 /* update the playlist on the jukebox
  */
 - (void)updateJukeboxPlaylist:(Playlist *)playlist
@@ -396,24 +260,23 @@
  */
 - (IBAction)addToPlaylist:(id)sender
 {
-	NSEnumerator *trackEnumerator = [njbTable selectedRowEnumerator];
-	NSNumber *index;
+	NSArray *selectedObjects = [arrayController selectedObjects];
+	NSEnumerator *trackEnumerator = [selectedObjects objectEnumerator];
+	Track *track;
 	Playlist *playlist = (Playlist *)[playlists objectAtIndex:[playlistsTable selectedRow]];
 	[playlist lock];
-	while (index = [trackEnumerator nextObject])
+	while (track = (Track *)[trackEnumerator nextObject])
 	{
 		// this is added at the end so don't worry about indexes changing
-		Track *track = (Track *)[[self itemArrayDisplaying] objectAtIndex:[index unsignedIntValue]];
 		[playlist addTrack:track];
 	}
 	[playlist unlock];
 	// update the jukebox
-	NSLog(@"Updating playlist from addToPlaylist:");
 	[self updateJukeboxPlaylist:playlist];
 	[playlistsTrackTable reloadData];
 }
 
-/* delete the selected tracks in njbTable from the selected
+/* delete the selected tracks in playlistsTrackTable from the selected
  * playlist
  */
 - (IBAction)deleteFromPlaylist:(id)sender
@@ -422,7 +285,7 @@
 	NSNumber *index;
 	Playlist *playlist = (Playlist *)[playlists objectAtIndex:[playlistsTable selectedRow]];
 	NSMutableArray *tracksToDelete = [[NSMutableArray alloc] initWithCapacity:[playlistsTrackTable numberOfSelectedRows]];
-	MyMutableArray *tracks = [playlist tracks];
+	NSMutableArray *tracks = [playlist tracks];
 	while (index = [trackEnumerator nextObject])
 	{
 		[tracksToDelete addObject:[tracks objectAtIndex:[index unsignedIntValue]]];
@@ -431,7 +294,6 @@
 	[playlist deleteTracks:tracksToDelete];
 	[playlist unlock];
 	// update the jukebox
-	NSLog(@"Updating playlist from deleteFromPlaylist:");
 	[self updateJukeboxPlaylist:playlist];
 	[tracksToDelete release];
 	[playlistsTrackTable reloadData];
@@ -462,7 +324,6 @@
 	
 	int row = [playlistsTable selectedRow];
 	// update the jukebox first so not released
-	NSLog(@"Updating playlist from deletePlaylist:");
 	[self deleteJukeboxPlaylist:(Playlist *)[playlists objectAtIndex:row]];
 	[playlists removeObjectAtIndex:row];
 	tracksInCurrentPlaylist = nil;
@@ -472,6 +333,8 @@
 
 - (void)NJBConnected:(NSNotification *)note
 {
+	trackListUpToDate = NO;
+	
 	tracksInCurrentPlaylist = nil;
 	[playlists release];
 	playlists = nil;
@@ -488,6 +351,8 @@
 
 - (void)NJBDisconnected:(NSNotification *)note
 {
+	trackListUpToDate = NO;
+	
 	[newPlaylistButton setEnabled:NO];
 	[deletePlaylistButton setEnabled:NO];
 	[addToPlaylistButton setEnabled:NO];
@@ -495,8 +360,6 @@
 	[moveTracksUpButton setEnabled:NO];
 	[moveTracksDownButton setEnabled:NO];
 	[super NJBDisconnected:note];
-	
-	trackListIsUpToDate = NO;
 }
 
 - (void)tableView:(NSTableView *)aTableView setObjectValue:(id)anObject forTableColumn:(NSTableColumn *)aTableColumn row:(int)rowIndex
@@ -508,12 +371,12 @@
 	[playlist setName:anObject];
 	[playlist unlock];
 	// update the jukebox
-	NSLog(@"Updating playlist from tableView:setObjectValue:forTableColumn:row:");
 	[self updateJukeboxPlaylist:playlist];
 }
 
 - (void)tableViewSelectionDidChange:(NSNotification *)aNotification
 {
+	[super tableViewSelectionDidChange:aNotification];
 	// don't necessarily have to disable all buttons if not connected
 	// as will have been disabled in NJBDisconnected:
 	if ([aNotification object] == njbTable)
@@ -691,4 +554,9 @@
 	[playlistsTrackTable selectRow:(index + 1) byExtendingSelection:YES];
 }
 
+- (void)NJBTrackListModified:(NSNotification *)note
+{
+	trackListUpToDate = NO;
+}
+
 @end
