@@ -43,4 +43,13 @@
 - (void)setDrawerEdge:(NSRectEdge)drawerEdge;
 - (BOOL)writeTagAfterCopy;
 - (void)setWriteTagAfterCopy:(BOOL)writeTagAfterCopy;
+- (int)lengthTol;
+- (void)setLengthTol:(int)lengthTol;
+- (int)filesizeTol;
+- (void)setFilesizeTol:(int)filesizeTol;
+- (int)titleTol;
+- (void)setTitleTol:(int)titleTol;
+- (int)artistTol;
+- (void)setArtistTol:(int)artistTol;
+
 @end
