@@ -15,6 +15,7 @@
 // declare the private methods
 @interface PreferencesWindowController (PrivateAPI)
 - (void)displayPreferences;
+- (BOOL)isDirty;
 @end
 
 @implementation PreferencesWindowController
@@ -63,9 +64,6 @@
  */
 - (void)displayPreferences
 {
-	// start with no changes
-	isDirty = NO;
-	
 	[self setStateForButton:connectOnStartup fromBool:[preferences connectOnStartup]];
 	[self setStateForButton:writeTagAfterCopy fromBool:[preferences writeTagAfterCopy]];
 	[self setStateForButton:startupTabLastUsed fromBool:[preferences startupTabLastUsed]];
@@ -103,6 +101,20 @@
 		[dataTabDir setEnabled:NO];
 	}
 	[filenameFormat setStringValue:[preferences filenameFormat]];
+	
+	[duplicatesLengthTol setIntValue:[preferences lengthTol]];
+	[duplicatesLengthTolStepper setIntValue:[preferences lengthTol]];
+	[duplicatesFilesizeTol setIntValue:[preferences filesizeTol]];
+	[duplicatesFilesizeTolStepper setIntValue:[preferences filesizeTol]];
+	[duplicatesTitleTol setIntValue:[preferences titleTol]];
+	[duplicatesTitleTolStepper setIntValue:[preferences titleTol]];
+	[duplicatesArtistTol setIntValue:[preferences artistTol]];
+	[duplicatesArtistTolStepper setIntValue:[preferences artistTol]];
+	
+	// start with no changes
+	// this goes at the end as setting text above causes boxes to call controlTextDidChange
+	isDirty = NO;
+	
 }
 
 /* Sets the button state mapping YES->NSOnState, NO->NSOffState
@@ -164,6 +176,11 @@
 	}
 	[preferences setFilenameFormat:[filenameFormat stringValue]];
 	
+	[preferences setLengthTol:[duplicatesLengthTol intValue]];
+	[preferences setFilesizeTol:[duplicatesFilesizeTol intValue]];
+	[preferences setTitleTol:[duplicatesTitleTol intValue]];
+	[preferences setArtistTol:[duplicatesArtistTol intValue]];
+	
 	[self close];
 }
 
@@ -282,7 +299,7 @@
 
 - (BOOL)windowShouldClose:(id)sender
 {
-	if (isDirty)
+	if ([self isDirty])
 	{
 		int result = NSRunAlertPanel(@"Save Preferences", @"Do you want to save changes to preferences?", @"Yes", @"No", @"Cancel");
 		if (result == NSAlertDefaultReturn)
@@ -326,4 +343,17 @@
 	isDirty = YES;
 }
 
+- (BOOL)isDirty
+{
+	if (isDirty)
+		return YES;
+	// check to see if any have changed: easier than finding out when stepper clicked
+	if ([preferences lengthTol] != [duplicatesLengthTol intValue] ||
+			[preferences filesizeTol] != [duplicatesFilesizeTol intValue] ||
+			[preferences titleTol] != [duplicatesTitleTol intValue] ||
+			[preferences artistTol] != [duplicatesArtistTol intValue])
+		return YES;
+	return NO;
+}
+
 @end
