@@ -14,6 +14,7 @@
 #define COLUMN_TYPE @"Type"
 #define COLUMN_DESCRIPTION @"Description"
 #define COLUMN_STATUS @"Status"
+#define COLUMN_RESULT @"Result"
 
 @implementation QueueTab
 
@@ -62,9 +63,6 @@
 	nc = [NSNotificationCenter defaultCenter];
 	[nc addObserver:self selector:@selector(taskComplete:) name:NOTIFICATION_TASK_COMPLETE object:nil];
 	[nc addObserver:self selector:@selector(taskStarting:) name:NOTIFICATION_TASK_STARTING object:nil];
-	
-	[queueTable setAutosaveName:PREF_KEY_QUEUE_TAB_TABLE];
-	[queueTable setAutosaveTableColumns:YES];
 }
 
 /* should be called when an item is added to the 
@@ -145,6 +143,8 @@
 		
 		return nstr; 
   }
+	else if ([ident isEqual:COLUMN_RESULT])
+		return [[queueItems objectAtIndex:rowIndex] resultString];
 	else
 	{
 		NSLog(@"Invalid column tag in QueueTab");
@@ -177,9 +177,7 @@
 /* called when a task finishes
  */
 - (void)taskComplete:(NSNotification *)note
-{
-	NSLog(@"taskComplete");
-	
+{	
 	if (currentTaskIndex >= [queueItems count])
 	{
 		NSLog(@"Error in QueueTab: currentTaskIndex is too big!");
@@ -188,10 +186,13 @@
 	
 	QueueTabItem *currentItem = [queueItems objectAtIndex:currentTaskIndex];
 	
+	NJBTransactionResult *result = [note object];
+	
+	[currentItem setResultString:[result resultString]];
+	
 	if ([currentItem status] != ITEM_STATUS_CANCELLED)
 	{
-		BOOL success = [[note object] boolValue];
-		if (success)
+		if ([result success])
 			[currentItem setStatus:ITEM_STATUS_FINISHED_WITH_SUCCESS];
 		else
 			[[queueItems objectAtIndex:currentTaskIndex] setStatus:ITEM_STATUS_FINISHED_WITH_FAILURE];
@@ -202,9 +203,7 @@
 /* called when a task starts
  */
 - (void)taskStarting:(NSNotification *)note
-{
-	NSLog(@"taskStarting");
-	
+{	
 	currentTaskIndex++;
 	while ([[queueItems objectAtIndex:currentTaskIndex] status] == ITEM_STATUS_CANCELLED)
 	{
