@@ -9,7 +9,7 @@
 #import "MyItem.h"
 
 // these should be in numerical order of sorting that we'd like compareByCodec to use
-typedef enum _codecTypes {CODEC_UNDEFINED = -1, CODEC_MP3 = 0, CODEC_WMA = 1, CODEC_WAV = 2} codecTypes;
+typedef enum _codecTypes {CODEC_UNDEFINED = -1, CODEC_MP3 = 0, CODEC_WMA = 1, CODEC_WAV = 2, CODEC_AA = 3} codecTypes;
 
 @interface Track : MyItem {
 @private
@@ -24,8 +24,6 @@ typedef enum _codecTypes {CODEC_UNDEFINED = -1, CODEC_MP3 = 0, CODEC_WMA = 1, CO
 	unsigned year;
 	codecTypes codec;
 	unsigned filesize;
-	// the njb track id for this track
-	unsigned trackID;
 }
 - (id)initWithTrack:(Track *)track;
 - (NSString *)title;
@@ -52,29 +50,11 @@ typedef enum _codecTypes {CODEC_UNDEFINED = -1, CODEC_MP3 = 0, CODEC_WMA = 1, CO
 - (NSString *)njbCodec;
 - (void)setCodec:(codecTypes)newCodec;
 - (unsigned)filesize;
-- (void)setTrackID:(unsigned)newTrackID;
-- (unsigned)trackID;
 - (NSString *)filesizeString;
 - (void)setFilesize:(unsigned)newFilesize;
+- (void)setValuesToTrack:(Track *)track;
 
-- (NSComparisonResult)compareByTitle:(Track *)other;
-+ (SEL)compareByTitleSelector;
-- (NSComparisonResult)compareByAlbum:(Track *)other;
-+ (SEL)compareByAlbumSelector;
-- (NSComparisonResult)compareByArtist:(Track *)other;
-+ (SEL)compareByArtistSelector;
-- (NSComparisonResult)compareByGenre:(Track *)other;
-+ (SEL)compareByGenreSelector;
 - (NSComparisonResult)compareByLength:(Track *)other;
-+ (SEL)compareByLengthSelector;
-- (NSComparisonResult)compareByFilesize:(Track *)other;
-+ (SEL)compareByFilesizeSelector;
-- (NSComparisonResult)compareByCodec:(Track *)other;
-+ (SEL)compareByCodecSelector;
-- (NSComparisonResult)compareByTrackNumber:(Track *)other;
-+ (SEL)compareByTrackNumberSelector;
-- (NSComparisonResult)compareByYear:(Track *)other;
-+ (SEL)compareByYearSelector;
-
+- (NSComparisonResult)compareUnsignedInts:(unsigned)mine withOther:(unsigned)theirs;
 
 @end
