@@ -14,10 +14,18 @@
 #define __LIBNJB__H
 
 /** The version of this installation of libnjb */
-#define LIBNJB_VERSION 2.0
+#define LIBNJB_VERSION 2.2.3
 /** A legacy definition - nowadays we always compile for libusb */
 #define LIBNJB_COMPILED_FOR_LIBUSB 1
 
+/* This handles MSVC pecularities */
+#ifdef _MSC_VER
+#include <windows.h>
+#define __WIN32__
+#define snprintf _snprintf
+#define ssize_t SSIZE_T
+#endif
+
 #include <sys/types.h>
 #ifdef __WIN32__
 /* Windows specific code, types that do not exist in Windows
@@ -25,12 +33,25 @@
  */
 typedef char int8_t;
 typedef unsigned char u_int8_t;
-typedef short int16_t;
-typedef unsigned short u_int16_t;
-typedef int int32_t;
-typedef unsigned int u_int32_t;
+typedef __int16 int16_t;
+typedef unsigned __int16 u_int16_t;
+typedef __int32 int32_t;
+typedef unsigned __int32 u_int32_t;
 typedef unsigned __int64 u_int64_t;
 #endif
+#ifdef __sun
+
+/*
+ * Solaris specific code, types that do not exist in Solaris'
+ * sys/types.h. u_intN_t is the ISO C way, whereas Solaris'
+ * way is POSIXly correct.
+ */
+#define u_int8_t uint8_t
+#define u_int16_t uint16_t
+#define u_int32_t uint32_t
+#define u_int64_t uint64_t
+#endif
+
 
 #include <stdio.h>
 #include <usb.h>
@@ -60,17 +81,7 @@ typedef unsigned __int64 u_int64_t;
 #define NJB_DEVICE_NJBZENMICRO	0x09 /**< Nomad Jukebox Zen Micro */
 #define NJB_DEVICE_DELLDJ2	0x0a /**< Second Generation Dell DJ */
 #define NJB_DEVICE_POCKETDJ     0x0b /**< Dell Pocket DJ */
-/** @} */
-
-/**
- * @defgroup power Power connection and status
- * @{
- */
-#define NJB_POWER_BATTERY       0x00 /**< If the device is running on battery (NJB1 only) */
-#define NJB_POWER_AC_CHARGED    0x01 /**< If the device is running on DC 
-					  and the battery is fully charged */
-#define NJB_POWER_AC_CHARGING   0x03 /**< If the device is running on DC
-					  and the battery is being charged */
+#define NJB_DEVICE_ZENSLEEK	0x0c /**< Zen Sleek */
 /** @} */
 
 /**
@@ -89,6 +100,7 @@ typedef unsigned __int64 u_int64_t;
 #define NJB_CODEC_MP3	"MP3" /**< The MPEG audio level 3 codec */
 #define NJB_CODEC_WMA	"WMA" /**< The Windows Media Audio codec */
 #define NJB_CODEC_WAV	"WAV" /**< The RIFF waveform codec */
+#define NJB_CODEC_AA    "AA"  /**< The Audible.com codec */
 /** @} */
 
 /**
@@ -142,7 +154,6 @@ typedef unsigned char owner_string[OWNER_STRING_LENGTH + 1];
  * @{
  */
 typedef struct njb_struct njb_t; /**< @see njb_struct */
-typedef struct njbid_struct njbid_t; /**< See struct definition */
 typedef struct njb_songid_frame_struct njb_songid_frame_t; /**< See struct definition */
 typedef struct njb_songid_struct njb_songid_t; /**< See struct definition */
 typedef struct njb_playlist_track_struct njb_playlist_track_t; /**< See struct definition */
@@ -157,7 +168,6 @@ typedef struct njb_keyval_struct njb_keyval_t; /**< See struct definition */
  * Main NJB object struct
  */
 struct njb_struct {
-	njbid_t *njbid; /**< The ID for this jukebox */
 	struct usb_device *device; /**< The libusb device for this jukebox */
 	usb_dev_handle *dev; /**< The libusb device handle for this jukebox */
 	u_int8_t usb_config; /**< The libusb config for this jukebox */
@@ -171,22 +181,6 @@ struct njb_struct {
 	void *error_stack; /**< Error stack, used inside libnjb */
 };
 
-/**
- * Jukebox identification struct
- */
-struct njbid_struct {
-	unsigned char id[16]; /**< The SDMI device ID for this device, 16 bytes */
-	u_int8_t fwMajor; /**< Firmware major revision */
-	u_int8_t fwMinor; /**< Firmware minor revision */
-	u_int8_t fwRel; /**< Firmware release */
-	u_int8_t hwMajor; /**< Hardware major revision */
-	u_int8_t hwMinor; /**< Hardware minor revision */
-	u_int8_t hwRel; /**< Hardware release */
-	char productName[33]; /**< A string with the product name */
-	u_int8_t power; /**< If the power cord is plugged in or not */
-};
-
-
 /* Song/track tag definitions */
 
 /**
@@ -283,7 +277,8 @@ struct njb_datafile_struct {
 	/**< This is an ordinary UNIX styled timestamp for the file. */
 	u_int32_t flags;
 	/**< 
-	* These are ordinary windows file flags: 
+	 * These are ordinary windows file flags: 
+	 * <pre>
 	 * bit (from MSB)   meaning
 	 * -----------------------------
 	 * 31               Normal file (0x80000000U)
@@ -297,6 +292,7 @@ struct njb_datafile_struct {
 	 * 19               Compressed file (0x00080000U)
 	 * 17               Sparse file (0x00020000U)
 	 * 16               Temporary file (0x00010000U)
+	 * </pre>
 	 *
 	 * SAMBA maps the bits to Unix permissions thus:
 	 *
@@ -325,6 +321,16 @@ struct njb_datafile_struct {
 };
 
 
+/**
+ * This is the EAX Control Type
+ */
+typedef enum {
+       NJB_EAX_NO_CONTROL,
+       NJB_EAX_SLIDER_CONTROL,
+       NJB_EAX_FIXED_OPTION_CONTROL
+} njb_eax_control_t;
+
+
 /**
  * This is the EAX API type.
  */
@@ -345,49 +351,33 @@ struct njb_eax_struct {
 	 * effects appearing after each other are grouped if their 
 	 * group numbers are identical
 	 */
-	u_int8_t selectable;
-	/**< 
-	 * 0x00 = not selectable, 0x01 = selectable
-	 * If this effect has several different settings
-	 */
-	u_int16_t current_selectionvalue;
-	/**< 
-	 * The currently selected patch, if this is a selectable effect
-	 * with several patches.
-	 */
-	u_int16_t min_selectionvalue;
+        njb_eax_control_t type;
+        /**<
+         * NJB_EAX_NO_CONTROL,
+         * NJB_EAX_SLIDER_CONTROL,
+         * NJB_EAX_FIXED_OPTION_CONTROL
+         *
+         * this replaces selectable, scalable with a single variable
+         */
+        int16_t   current_value;
 	/**<
-	 * The minimal selectable patch index, if this is a selectable
-	 * effect.
-	 */
-	u_int16_t max_selectionvalue;
-	/**<
-	 * The maximum selectable patch index, if this is a selectable
-	 * effect.
-	 */
-	char **selection_names;
-	/**< 
-	 * Array with names for the selections, if this is a selectable
-	 * effect.
-	 */
-	u_int8_t scaleable;
-	/**< 
-	 * 0x00 = not scalable, 0x01 = scalable
-	 * Tells if this effect can be slided on a scale.
-	 */
-	int16_t current_scalevalue;
-	/**<
-	 * The current scale value, if this value is scaleable.
+	 * The current value of this effect.
 	 * Notice that this value may be negative!
 	 */
-	int16_t min_scalevalue;
+        int16_t   min_value;
 	/**<
-	 * The minumum scale value, if this value is scaleable.
+	 * The minumum value for this effect.
 	 * Notice that this value may be negative!
 	 */
-	int16_t max_scalevalue;
+        int16_t   max_value;
 	/**<
-	 * The maximum scale value, if this value is scaleable.
+	 * The maximum value for this effect.
+	 */
+	char **option_names;
+	/**< 
+	 * Array with names for the selections, if this is a fixed
+	 * option control. It may not be dereferenced for slider
+         * controls.
 	 */
 	njb_eax_t *next;
 	/**< 
@@ -454,8 +444,18 @@ void NJB_Ping(njb_t *njb);
 int NJB_Get_Disk_Usage (njb_t *njb, u_int64_t *btotal, u_int64_t *bfree);
 char *NJB_Get_Owner_String (njb_t *njb);
 int NJB_Set_Owner_String (njb_t *njb, const char *name);
-int NJB_Set_Bitmap(njb_t *njb, const char * pbm_file);
+int NJB_Get_Bitmap_Dimensions(njb_t *njb, int *x, int *y, int *bytes);
+int NJB_Set_Bitmap(njb_t *njb, const unsigned char *bitmap);
 njb_keyval_t *NJB_Get_Keys(njb_t *njb);
+u_int64_t NJB_Get_NJB1_Libcounter(njb_t *njb);
+int NJB_Send_Firmware (njb_t *njb, const char *path, NJB_Xfer_Callback *callback, void *data);
+int NJB_Get_Battery_Level (njb_t *njb);
+int NJB_Get_Battery_Charging (njb_t *njb);
+int NJB_Get_Auxpower (njb_t *njb);
+int NJB_Get_SDMI_ID(njb_t *njb, u_int8_t *sdmiid);
+const char *NJB_Get_Device_Name(njb_t *njb, int type);
+int NJB_Get_Firmware_Revision(njb_t *njb, u_int8_t *major, u_int8_t *minor, u_int8_t *release);
+int NJB_Get_Hardware_Revision(njb_t *njb, u_int8_t *major, u_int8_t *minor, u_int8_t *release);
 /**
  * @}
  * @defgroup tagapi The track and tag (song ID metadata) manipulation API
@@ -467,11 +467,12 @@ void NJB_Songid_Addframe(njb_songid_t *song, njb_songid_frame_t *frame);
 void NJB_Songid_Reset_Getframe(njb_songid_t *song);
 njb_songid_frame_t *NJB_Songid_Getframe(njb_songid_t *song);
 njb_songid_frame_t *NJB_Songid_Findframe(njb_songid_t *song, const char *label);
-njb_songid_frame_t *NJB_Songid_Frame_New_String(char *label, char *value);
-njb_songid_frame_t *NJB_Songid_Frame_New_Uint16(char *label, u_int16_t value);
-njb_songid_frame_t *NJB_Songid_Frame_New_Uint32(char *label, u_int32_t value);
+njb_songid_frame_t *NJB_Songid_Frame_New_String(const char *label, const char *value);
+njb_songid_frame_t *NJB_Songid_Frame_New_Uint16(const char *label, u_int16_t value);
+njb_songid_frame_t *NJB_Songid_Frame_New_Uint32(const char *label, u_int32_t value);
 /* Good helper functions for creating frames - USE THESE IF YOU CAN */
-#define NJB_Songid_Frame_New_Codec(a) NJB_Songid_Frame_New_String(FR_CODEC, a)
+njb_songid_frame_t *NJB_Songid_Frame_New_Codec(const char *value);
+/* #define NJB_Songid_Frame_New_Codec(a) NJB_Songid_Frame_New_String(FR_CODEC, a) */
 #define NJB_Songid_Frame_New_Title(a) NJB_Songid_Frame_New_String(FR_TITLE, a)
 #define NJB_Songid_Frame_New_Album(a) NJB_Songid_Frame_New_String(FR_ALBUM, a)
 #define NJB_Songid_Frame_New_Genre(a) NJB_Songid_Frame_New_String(FR_GENRE, a)
@@ -518,6 +519,7 @@ void NJB_Playlist_Reset_Gettrack(njb_playlist_t *pl);
 njb_playlist_track_t *NJB_Playlist_Gettrack(njb_playlist_t *pl);
 int NJB_Playlist_Set_Name(njb_playlist_t *pl, const char *name);
 void NJB_Playlist_Deltrack(njb_playlist_t *pl, unsigned int pos);
+void NJB_Playlist_Deltrack_TrackID(njb_playlist_t *pl, u_int32_t trackid);
 njb_playlist_track_t *NJB_Playlist_Track_New(u_int32_t trackid);
 void NJB_Playlist_Track_Destroy(njb_playlist_track_t *track);
 /**
